@@ -55,6 +55,9 @@
 #include "ShowSelectionOnlyManager.h"
 #include "enums.h"
 #include "MainWindow/DirtyIndicator.h"
+#include <qtooltip.h>
+
+using Utilities::StringSet;
 
 AnnotationDialog::Dialog::Dialog( QWidget* parent, const char* name )
     : QDialog( parent, name ), _viewer(0)
@@ -141,11 +144,13 @@ AnnotationDialog::Dialog::Dialog( QWidget* parent, const char* name )
     _prevBut->setIconSet( KGlobal::iconLoader()->loadIconSet( QString::fromLatin1( "1leftarrow" ), KIcon::Desktop, 22 ) );
     _prevBut->setFixedWidth( 40 );
     lay6->addWidget( _prevBut );
+    QToolTip::add( _prevBut, i18n("Annotate previous image") );
 
     _nextBut = new QPushButton( top2 );
     _nextBut->setIconSet( KGlobal::iconLoader()->loadIconSet( QString::fromLatin1( "1rightarrow" ), KIcon::Desktop, 22 ) );
     _nextBut->setFixedWidth( 40 );
     lay6->addWidget( _nextBut );
+    QToolTip::add( _nextBut, i18n("Annotate next image") );
 
     lay6->addStretch(1);
 
@@ -153,17 +158,27 @@ AnnotationDialog::Dialog::Dialog( QWidget* parent, const char* name )
     lay6->addWidget( _rotateLeft );
     _rotateLeft->setIconSet( KGlobal::iconLoader()->loadIconSet( QString::fromLatin1( "rotate_ccw" ), KIcon::Desktop, 22 ) );
     _rotateLeft->setFixedWidth( 40 );
+    QToolTip::add( _rotateLeft, i18n("Rotate counterclockwise") );
 
     _rotateRight = new QPushButton( top2 );
     lay6->addWidget( _rotateRight );
     _rotateRight->setIconSet( KGlobal::iconLoader()->loadIconSet( QString::fromLatin1( "rotate_cw" ), KIcon::Desktop, 22 ) );
     _rotateRight->setFixedWidth( 40 );
+    QToolTip::add( _rotateRight, i18n("Rotate clockwise") );
 
+    _copyPreviousBut = new QPushButton( top2 );
+    lay6->addWidget( _copyPreviousBut );
+    _copyPreviousBut->setIconSet( KGlobal::iconLoader()->loadIconSet( QString::fromLatin1( "legalmoves" ), KIcon::Desktop, 22 ) );
+    _copyPreviousBut->setFixedWidth( 40 );
+    connect( _copyPreviousBut, SIGNAL( clicked() ), this, SLOT( slotCopyPrevious() ) );
+    QToolTip::add( _copyPreviousBut, i18n("Copy tags from previously tagged image") );
+    
     lay6->addStretch( 1 );
     _delBut = new QPushButton( top2 );
     _delBut->setPixmap( KGlobal::iconLoader()->loadIcon( QString::fromLatin1( "editdelete" ), KIcon::Desktop, 22 ) );
     lay6->addWidget( _delBut );
     connect( _delBut, SIGNAL( clicked() ), this, SLOT( slotDeleteImage() ) );
+    QToolTip::add( _delBut, i18n("Delete image") );
 
     lay6->addStretch(1);
 
@@ -322,8 +337,28 @@ void AnnotationDialog::Dialog::slotOK()
     qApp->eventLoop()->exitLoop();
 
     // I shouldn't emit changed before I've actually commited the changes, otherwise the listeners will act on the old data.
-    if ( anyChanges )
+    if ( anyChanges ) {
         MainWindow::DirtyIndicator::markDirty();
+        _thumbnailTextShouldReload = true;
+    }
+}
+
+/*
+ * Copy tags (only tags/categories, not description/label/...) from previous image to the currently showed one
+ */
+void AnnotationDialog::Dialog::slotCopyPrevious()
+{
+    if ( _setup != InputSingleImageConfigMode )
+        return;
+    if ( _current < 1 )
+        return;
+
+    // FIXME: it would be better to compute the "previous image" in a better way, but let's stick with this for now...
+    DB::ImageInfo& old_info = _editList[ _current - 1 ];
+
+    for( QPtrListIterator<ListSelect> it( _optionList ); *it; ++it ) {
+        (*it)->setSelection( old_info.itemsOfCategory( (*it)->category() ) );
+    }
 }
 
 void AnnotationDialog::Dialog::load()
@@ -355,6 +390,7 @@ void AnnotationDialog::Dialog::load()
 
     _nextBut->setEnabled( _current != (int)_origList.count()-1 );
     _prevBut->setEnabled( _current != 0 );
+    _copyPreviousBut->setEnabled( _current != 0 );
 
     _preview->setImage( info );
 
@@ -427,9 +463,11 @@ int AnnotationDialog::Dialog::configure( DB::ImageInfoList list, bool oneAtATime
 
         _prevBut->setEnabled( false );
         _nextBut->setEnabled( false );
+        _copyPreviousBut->setEnabled( false );
     }
 
     _thumbnailShouldReload = false;
+    _thumbnailTextShouldReload = false;
 
     showHelpDialog( oneAtATime ? InputSingleImageConfigMode : InputMultiImageConfigMode );
     return exec();
@@ -471,7 +509,7 @@ void AnnotationDialog::Dialog::setup()
         _revertBut->hide();
         setCaption( i18n("Search") );
         loadInfo( _oldSearch );
-        _preview->setImage( locate("data", QString::fromLatin1("kphotoalbum/pics/search.jpg") ) );
+        _preview->setImage(Utilities::locateDataFile(QString::fromLatin1("pics/search.jpg")));
         _nextBut->setEnabled( false );
         _prevBut->setEnabled( false );
         _rotateLeft->setEnabled( false );
@@ -483,13 +521,14 @@ void AnnotationDialog::Dialog::setup()
         _revertBut->show();
         setCaption( i18n("Annotations") );
         if ( _setup == InputMultiImageConfigMode ) {
-            _preview->setImage( locate("data", QString::fromLatin1("kphotoalbum/pics/multiconfig.jpg") ) );
+            _preview->setImage(Utilities::locateDataFile(QString::fromLatin1("pics/multiconfig.jpg")));
         }
         _rotateLeft->setEnabled( true );
         _rotateRight->setEnabled( true );
     }
 
     _delBut->setEnabled( _setup == InputSingleImageConfigMode );
+    _copyPreviousBut->setEnabled( _setup == InputSingleImageConfigMode );
 
     for( QPtrListIterator<ListSelect> it( _optionList ); *it; ++it )
         (*it)->setMode( _setup );
@@ -660,7 +699,7 @@ void AnnotationDialog::Dialog::slotRenameOption( DB::Category* category, const Q
 void AnnotationDialog::Dialog::reject()
 {
     if ( hasChanges() ) {
-        int code =  KMessageBox::questionYesNo( this, i18n("<p>Changes made to current item, really cancel?</p>") );
+        int code =  KMessageBox::questionYesNo( this, i18n("<p>Some changes are made to annotations. Do you really want to cancel all recent changes for each affected file?</p>") );
         if ( code == KMessageBox::No )
             return;
     }
@@ -729,6 +768,11 @@ bool AnnotationDialog::Dialog::thumbnailShouldReload() const
     return _thumbnailShouldReload;
 }
 
+bool AnnotationDialog::Dialog::thumbnailTextShouldReload() const
+{
+    return _thumbnailTextShouldReload;
+}
+
 void AnnotationDialog::Dialog::slotAddTimeInfo()
 {
     _addTime->hide();
@@ -923,8 +967,11 @@ void AnnotationDialog::Dialog::setupActions()
     new KAction( i18n("Delete"), CTRL+Key_Delete, this, SLOT( slotDeleteImage() ),
                  _actions, "annotationdialog-delete-image" );
 
-    new KAction( i18n("Rotate Left"), 0, this, SLOT( rotateLeft() ), _actions, "annotationdialog-rotate-left" );
-    new KAction( i18n("Rotate Right"), 0, this, SLOT( rotateRight() ), _actions, "annotationdialog-rotate-right" );
+    new KAction( i18n("Copy tags from previous image"), CTRL+Key_Insert, this, SLOT( slotCopyPrevious() ),
+                 _actions, "annotationdialog-copy-previous");
+
+    new KAction( i18n("Rotate counterclockwise"), 0, this, SLOT( rotateLeft() ), _actions, "annotationdialog-rotate-left" );
+    new KAction( i18n("Rotate clockwise"), 0, this, SLOT( rotateRight() ), _actions, "annotationdialog-rotate-right" );
 
     connect( _nextBut, SIGNAL( clicked() ), this, SLOT( slotNext() ) );
     connect( _prevBut, SIGNAL( clicked() ), this, SLOT( slotPrev() ) );
