@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2005 Jesper K. Pedersen <blackie@kde.org>
+/* Copyright (C) 2003-2006 Jesper K. Pedersen <blackie@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -19,35 +19,26 @@
 #ifndef TYPEFOLDER_H
 #define TYPEFOLDER_H
 #include "Folder.h"
+#include "DB/Category.h"
 
 namespace Browser
 {
+class BrowserItem;
 
 class TypeFolder :public Folder {
 public:
-    TypeFolder( const QString& category, const ImageSearchInfo& info, Browser* parent );
+    TypeFolder( const DB::CategoryPtr& category, const DB::ImageSearchInfo& info, BrowserWidget* parent );
     virtual FolderAction* action( bool ctrlDown = false );
     virtual QPixmap pixmap();
     virtual QString text() const;
-    virtual QString countLabel() const;
-private:
-    QString _category;
-};
-
-class TypeFolderAction :public FolderAction {
-
-public:
-    TypeFolderAction( const QString& category, const ImageSearchInfo& info, Browser* parent  );
-    virtual void action( BrowserItemFactory* factory );
-    virtual bool showsImages() const { return false; }
-    virtual bool contentView() const;
-    virtual QString title() const;
-    virtual QString category() const;
+    virtual QString imagesLabel() const;
+    virtual QString videosLabel() const;
 
 private:
-    QString _category;
+    const DB::CategoryPtr _category;
 };
 
+
 }
 
 #endif /* TYPEFOLDER_H */
