@@ -26,17 +26,17 @@
 namespace DB
 {
 
+
 class NewImageFinder
 {
 public:
     bool findImages();
-    bool calculateMD5sums( const QStringList& list );
+    bool calculateMD5sums( const QStringList& list, DB::MD5Map* map, bool* wasCanceled = 0 );
 
 protected:
     void searchForNewFiles( const QDict<void>& loadedFiles, QString directory );
     void loadExtraFiles();
     ImageInfoPtr loadExtraFile( const QString& name, DB::MediaType type );
-    QString MD5Sum( const QString& fileName );
 
 private:
     typedef QValueList< QPair< QString, DB::MediaType > > LoadList;
