@@ -36,16 +36,17 @@ public:
 protected slots:
     void readInfo();
     void showFileList();
-    void warnAboutDates( bool );
+    bool warnAboutChanges();
 
 private:
     QStringList _list;
-    QLabel* _label;
+    QLabel* _title;
     QCheckBox* _exifDB;
     QCheckBox* _date;
     QCheckBox* _orientation;
+    QCheckBox* _label;
     QCheckBox* _description;
-    QCheckBox* _force_date;
+    QCheckBox* _categories;
 };
 
 }
