@@ -343,7 +343,7 @@ bool HTMLGenerator::Generator::generateContentPage( int width, int height, const
             continue;
 
         QString name = (*it)->name();
-        if ( info->itemsOfCategory( name ).count() != 0 && _setup.includeCategory(name) ) {
+        if ( !info->itemsOfCategory( name ).empty() && _setup.includeCategory(name) ) {
             QString val = QStringList(info->itemsOfCategory( name ).toList()).join( QString::fromLatin1(", ") );
             description += QString::fromLatin1("  <li> <b>%1:</b> %2\n").arg( name ).arg( val );
         }
@@ -379,7 +379,11 @@ QString HTMLGenerator::Generator::nameImage( const QString& fileName, int size )
     QString name = _nameMap[fileName];
     QString base = QFileInfo( name ).baseName(true);
     if ( size == maxImageSize() && !Utilities::isVideo( fileName ) )
-        return name;
+        if ( name.endsWith( QString::fromAscii(".jpg"), false ) ||
+                name.endsWith( QString::fromAscii(".jpeg"), false ) )
+            return name;
+        else
+            return base + QString::fromAscii(".jpg");
     else
         return QString::fromLatin1( "%1-%2.jpg" ).arg( base ).arg( size );
 }
