@@ -47,6 +47,9 @@
 #include "DB/ImageInfo.h"
 #include "MiniViewer.h"
 #include "XMLDB/Database.h"
+#include <kmessagebox.h>
+
+using Utilities::StringSet;
 
 class KPushButton;
 using namespace ImportExport;
@@ -77,7 +80,7 @@ void Import::imageImport( const KURL& url )
 }
 
 Import::Import( const KURL& url, QWidget* parent, const char* name )
-    :KWizard( parent, name, false ), _zip( 0 ), _hasFilled( false )
+    :KWizard( parent, name, false ), _zip( 0 ), _hasFilled( false ), _reportUnreadableFiles( true )
 {
     _kimFile = url;
     _tmp = new KTempFile( QString::null, QString::fromLatin1( ".kim" ) );
@@ -108,6 +111,7 @@ Import::Import( const QString& fileName, bool* ok, QWidget* parent, const char*
 {
     _kimFile.setPath( fileName );
     *ok = init( fileName );
+    connect( this, SIGNAL( failedToCopy( QStringList ) ), this, SLOT( aCopyFailed( QStringList ) ) );
 }
 
 bool Import::init( const QString& fileName )
@@ -299,7 +303,7 @@ void ImageRow::showImage()
 
             if( KIO::NetAccess::download( src, tmpFile, MainWindow::Window::theMainWindow() ) ) {
                 QImage img( tmpFile );
-                MiniViewer::show( img, _info );
+                MiniViewer::show( img, _info, static_cast<QWidget*>( parent() ) );
                 KIO::NetAccess::removeTempFile( tmpFile );
                 break;
             }
@@ -307,7 +311,7 @@ void ImageRow::showImage()
     }
     else {
         QImage img = QImage( _import->loadImage( _info->fileName(true) ) );
-        MiniViewer::show( img, _info );
+        MiniViewer::show( img, _info, static_cast<QWidget*>( parent() ) );
     }
 }
 
@@ -389,19 +393,16 @@ void Import::createCategoryPages()
 
 ImportMatcher* Import::createCategoryPage( const QString& myCategory, const QString& otherCategory )
 {
-    QStringList otherItems;
+    StringSet otherItems;
     DB::ImageInfoList images = selectedImages();
     for( DB::ImageInfoListConstIterator it = images.constBegin(); it != images.constEnd(); ++it ) {
-        DB::ImageInfoPtr info = *it;
-        StringSet items = info->itemsOfCategory( otherCategory );
-        for( StringSet::ConstIterator itemIt = items.begin(); itemIt != items.end(); ++itemIt )
-            otherItems.append( *itemIt );
+        otherItems += (*it)->itemsOfCategory( otherCategory );
     }
 
     QStringList myItems = DB::ImageDB::instance()->categoryCollection()->categoryForName( myCategory )->itemsInclCategories();
     myItems.sort();
 
-    ImportMatcher* matcher = new ImportMatcher( otherCategory, myCategory, otherItems, myItems, true, this, "import matcher" );
+    ImportMatcher* matcher = new ImportMatcher( otherCategory, myCategory, otherItems.toList(), myItems, true, this, "import matcher" );
     addPage( matcher, myCategory );
     return matcher;
 }
@@ -451,6 +452,12 @@ void Import::next()
 bool Import::copyFilesFromZipFile()
 {
     DB::ImageInfoList images = selectedImages();
+
+    _totalCopied = 0;
+    _progress = new QProgressDialog( i18n("Copying Images"), i18n("&Cancel"), 2 * images.count(), 0, "_progress", true );
+    _progress->setProgress( 0 );
+    _progress->show();
+
     for( DB::ImageInfoListConstIterator it = images.constBegin(); it != images.constEnd(); ++it ) {
         QString fileName = (*it)->fileName( true );
         QByteArray data = loadImage( fileName );
@@ -465,10 +472,20 @@ bool Import::copyFilesFromZipFile()
         QFile out( newName );
         if ( !out.open( IO_WriteOnly ) ) {
             KMessageBox::error( this, i18n("Error when writing image %s").arg( newName ) );
+            delete _progress;
+            _progress = 0;
             return false;
         }
         out.writeBlock( data, data.size() );
         out.close();
+
+        qApp->processEvents();
+        _progress->setProgress( ++_totalCopied );
+        if ( _progress->wasCanceled() ) {
+            delete _progress;
+            _progress = 0;
+            return false;
+        }
     }
     return true;
 }
@@ -477,7 +494,7 @@ void Import::copyFromExternal()
 {
     _pendingCopies = selectedImages();
     _totalCopied = 0;
-    _progress = new QProgressDialog( i18n("Copying Images"), i18n("&Cancel"), _pendingCopies.count(), 0, "_progress", true );
+    _progress = new QProgressDialog( i18n("Copying Images"), i18n("&Cancel"), 2 * _pendingCopies.count(), 0, "_progress", true );
     _progress->setProgress( 0 );
     _progress->show();
     connect( _progress, SIGNAL( canceled() ), this, SLOT( stopCopyingImages() ) );
@@ -491,6 +508,9 @@ void Import::copyNextFromExternal()
     QString fileName = info->fileName( true );
     KURL src1 = _kimFile;
     KURL src2 = _baseUrl + QString::fromLatin1( "/" );
+    bool succeeded = false;
+    QStringList tried;
+
     for ( int i = 0; i < 2; ++i ) {
         KURL src = src1;
         if ( i == 1 )
@@ -502,15 +522,44 @@ void Import::copyNextFromExternal()
             dest.setPath( Settings::SettingsData::instance()->imageDirectory() + _nameMap[fileName] );
             _job = KIO::file_copy( src, dest, -1, false, false, false );
             connect( _job, SIGNAL( result( KIO::Job* ) ), this, SLOT( aCopyJobCompleted( KIO::Job* ) ) );
+            succeeded = true;
             break;
-        }
+        } else
+            tried << src.prettyURL();
+    }
+
+    if (!succeeded)
+        emit failedToCopy( tried );
+}
+
+void Import::aCopyFailed( QStringList files )
+{
+    int result = _reportUnreadableFiles ?
+        KMessageBox::warningYesNoCancelList( _progress,
+            i18n("Can't copy file from any of the following locations:"),
+            files, QString::null, KStdGuiItem::cont(), KGuiItem( i18n("Continue without Asking") )) : KMessageBox::Yes;
+
+    switch (result) {
+        case KMessageBox::Cancel:
+            // This might be late -- if we managed to copy some files, we will
+            // just throw away any changes to the DB, but some new image files
+            // might be in the image directory...
+            deleteLater();
+            delete _progress;
+            _progress = 0;
+            break;
+
+        case KMessageBox::No:
+            _reportUnreadableFiles = false;
+            // fall through
+        default:
+            aCopyJobCompleted( 0 );
     }
 }
 
-
 void Import::aCopyJobCompleted( KIO::Job* job )
 {
-    if ( job->error() ) {
+    if ( job && job->error() ) {
         job->showErrorDialog( 0 );
         deleteLater();
         delete _progress;
@@ -554,6 +603,9 @@ void Import::slotFinish()
 
 void Import::updateDB()
 {
+    disconnect( _progress, SIGNAL( canceled() ), this, SLOT( stopCopyingImages() ) );
+    _progress->setLabelText( i18n("Updating Database") );
+
     // Run though all images
     DB::ImageInfoList images = selectedImages();
     for( DB::ImageInfoListConstIterator it = images.constBegin(); it != images.constEnd(); ++it ) {
@@ -565,7 +617,7 @@ void Import::updateDB()
         newInfo->setDate( info->date() );
         newInfo->rotate( info->angle() );
         newInfo->setDrawList( info->drawList() );
-        newInfo->setMD5Sum( info->MD5Sum() );
+        newInfo->setMD5Sum( Utilities::MD5Sum( newInfo->fileName(false) ) );
         DB::ImageInfoList list;
         list.append(newInfo);
         DB::ImageDB::instance()->addImages( list );
@@ -590,6 +642,10 @@ void Import::updateDB()
 
             }
         }
+
+        _progress->setProgress( ++_totalCopied );
+        if ( _progress->wasCancelled() )
+            break;
     }
     Browser::BrowserWidget::instance()->home();
 }
