@@ -40,6 +40,7 @@
 #include "Browser/BrowserWidget.h"
 #include "DB/ImageDB.h"
 #include "Utilities/Util.h"
+#include "Utilities/List.h"
 #include <kapplication.h>
 #include <ktip.h>
 #include <kprocess.h>
@@ -66,6 +67,7 @@
 #endif
 #ifdef HASEXIV2
 #  include "Exif/ReReadDialog.h"
+#  include "Exif/WriteDialog.h"
 #endif
 #include "ImageManager/ImageLoader.h"
 #include "SplashScreen.h"
@@ -94,7 +96,6 @@
 
 #ifdef SQLDB_SUPPORT
 #  include "SQLDB/Database.h"
-#  include "SQLDB/DatabaseHandler.h"
 #  include "SQLDB/ConfigFileHandler.h"
 #  include "SQLDB/QueryErrors.h"
 #  include <kexidb/kexidb_export.h>
@@ -103,6 +104,10 @@
 #include <kprogress.h>
 #include <krun.h>
 #include "DirtyIndicator.h"
+#include "Utilities/ShowBusyCursor.h"
+#include <kurldrag.h>
+#include <qclipboard.h>
+#include <stdexcept>
 
 MainWindow::Window* MainWindow::Window::_instance = 0;
 
@@ -216,13 +221,18 @@ MainWindow::Window::Window( QWidget* parent, const char* name )
     slotThumbNailSelectionChanged();
 }
 
+MainWindow::Window::~Window()
+{
+    DB::ImageDB::deleteInstance();
+}
+
 void MainWindow::Window::delayedInit()
 {
     SplashScreen* splash = SplashScreen::instance();
     setupPluginMenu();
 
     if ( Settings::SettingsData::instance()->searchForImagesOnStartup() ) {
-        splash->message( i18n("Searching for New Images and Videos") );
+        splash->message( i18n("Searching for New Files") );
         qApp->processEvents();
         DB::ImageDB::instance()->slotRescan();
     }
@@ -256,6 +266,7 @@ void MainWindow::Window::delayedInit()
 
     tellPeopleAboutTheVideos();
     checkIfAllFeaturesAreInstalled();
+
 }
 
 
@@ -350,6 +361,8 @@ void MainWindow::Window::configImages( const DB::ImageInfoList& list, bool oneAt
     _annotationDialog->configure( list,  oneAtATime );
     if ( _annotationDialog->thumbnailShouldReload() )
         reloadThumbnails(true);
+    else if ( _annotationDialog->thumbnailTextShouldReload() )
+        _thumbnailView->reload(false, false);
 }
 
 
@@ -367,7 +380,7 @@ void MainWindow::Window::createAnnotationDialog()
     if ( !_annotationDialog.isNull() )
         return;
 
-    _annotationDialog = new AnnotationDialog::Dialog( this,  "_annotationDialog" );
+    _annotationDialog = new AnnotationDialog::Dialog( 0,  "_annotationDialog" );
 }
 
 void MainWindow::Window::deleteAnnotationDialog()
@@ -386,6 +399,18 @@ void MainWindow::Window::slotSave()
     statusBar()->message(i18n("Saving... Done"), 5000 );
 }
 
+void MainWindow::Window::slotCopySelectedURLs()
+{
+    const QStringList& sel = selectedOnDisk();
+    KURL::List urls;
+
+    for (QStringList::const_iterator it = sel.begin(); it != sel.end(); ++it) {
+        urls.append( KURL( *it ) );
+    }
+
+    QApplication::clipboard()->setData( new KURLDrag( urls ) );
+}
+
 void MainWindow::Window::slotDeleteSelected()
 {
     if ( ! _deleteDialog )
@@ -397,7 +422,7 @@ void MainWindow::Window::slotDeleteSelected()
     DirtyIndicator::markDirty();
 
     QStringList images = _thumbnailView->imageList( ThumbnailView::ThumbnailWidget::SortedOrder );
-    Set<QString> allImages( DB::ImageDB::instance()->images() );
+    StringSet allImages( DB::ImageDB::instance()->images() );
     QStringList newSet;
     for( QStringList::Iterator it = images.begin(); it != images.end(); ++it ) {
         if ( allImages.contains( *it ) )
@@ -419,6 +444,16 @@ void MainWindow::Window::slotReReadExifInfo()
 #endif
 }
 
+void MainWindow::Window::slotWriteExifInfo()
+{
+#ifdef HASEXIV2
+    QStringList files = selectedOnDisk();
+    static Exif::WriteDialog* dialog = 0;
+    if ( ! dialog )
+        dialog = new Exif::WriteDialog( this );
+    dialog->exec( files );
+#endif
+}
 
 QStringList MainWindow::Window::selected( bool keepSortOrderOfDatabase )
 {
@@ -433,14 +468,18 @@ void MainWindow::Window::slotViewNewWindow()
     slotView( false, false );
 }
 
+/*
+ * Returns a list of files that are both selected and on disk. If there are no
+ * selected files, returns all files form current context that are on disk.
+ * */
 QStringList MainWindow::Window::selectedOnDisk()
 {
-    QStringList listOnDisk;
     QStringList list = selected();
     if ( list.count() == 0 )
-        list = DB::ImageDB::instance()->currentScope(  true );
+        return DB::ImageDB::instance()->currentScope( true );
 
-    for( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
+    QStringList listOnDisk;
+    for( QStringList::const_iterator it = list.constBegin(); it != list.constEnd(); ++it ) {
         if ( DB::ImageInfo::imageOnDisk( *it ) )
             listOnDisk.append( *it );
     }
@@ -450,19 +489,32 @@ QStringList MainWindow::Window::selectedOnDisk()
 
 void MainWindow::Window::slotView( bool reuse, bool slideShow, bool random )
 {
-    launchViewer( selectedOnDisk(), reuse, slideShow, random );
+    launchViewer( selected(), reuse, slideShow, random );
 }
 
 void MainWindow::Window::launchViewer( QStringList files, bool reuse, bool slideShow, bool random )
 {
+    int seek = -1;
     if ( files.count() == 0 ) {
-        QMessageBox::warning( MainWindow::Window::theMainWindow(), i18n("No Images or Videos to Display"),
-                              i18n("None of the selected images or videos were available on disk.") );
+        files = _thumbnailView->imageList( ThumbnailView::ThumbnailWidget::ViewOrder );
+    } else if ( files.count() == 1 ) {
+        // we fake it so it appears the user has selected all images
+        // and magically scrolls to the originally selected one
+        const QString fileName = ((const QStringList&)files).first();
+        files = _thumbnailView->imageList( ThumbnailView::ThumbnailWidget::ViewOrder );
+        seek = files.findIndex(fileName);
+    }
+
+    if ( !files.count() )
+        files = DB::ImageDB::instance()->currentScope( false );
+
+    if ( !files.count() ) {
+        KMessageBox::sorry( this, i18n("There are no images to be shown.") );
         return;
     }
 
     if (random)
-        files = Utilities::shuffle( files );
+        files = Utilities::shuffleList(files);
 
     Viewer::ViewerWidget* viewer;
     if ( reuse && Viewer::ViewerWidget::latest() ) {
@@ -474,7 +526,9 @@ void MainWindow::Window::launchViewer( QStringList files, bool reuse, bool slide
         viewer = new Viewer::ViewerWidget( "viewer" );
 
     viewer->show( slideShow );
-    viewer->load( files );
+    if (seek == -1)
+        seek = 0;
+    viewer->load( files, seek );
     viewer->raise();
 }
 
@@ -519,7 +573,7 @@ void MainWindow::Window::setupMenuBar()
     _generateHtml = new KAction( i18n("Generate HTML..."), 0, this, SLOT( slotExportToHTML() ), actionCollection(), "exportHTML" );
 
     new KAction( i18n( "Import..."), 0, this, SLOT( slotImport() ), actionCollection(), "import" );
-    new KAction( i18n( "Export..."), 0, this, SLOT( slotExport() ), actionCollection(), "export" );
+    new KAction( i18n( "Export/Copy Images..."), 0, this, SLOT( slotExport() ), actionCollection(), "export" );
 
 
     // Go menu
@@ -533,11 +587,14 @@ void MainWindow::Window::setupMenuBar()
 
     a = KStdAction::home( _browser, SLOT( home() ), actionCollection() );
 
+    a = KStdAction::redisplay( _browser, SLOT( go() ), actionCollection() );
+
     // The Edit menu
 #ifdef CODE_FOR_OLD_CUT_AND_PASTE_IN_THUMBNAIL_VIEW
     _cut = KStdAction::cut( _thumbNailViewOLD, SLOT( slotCut() ), actionCollection() );
     _paste = KStdAction::paste( _thumbNailViewOLD, SLOT( slotPaste() ), actionCollection() );
 #endif
+    KStdAction::copy( this, SLOT( slotCopySelectedURLs() ), actionCollection() );
     _selectAll = KStdAction::selectAll( _thumbnailView, SLOT( selectAll() ), actionCollection() );
     KStdAction::find( this, SLOT( slotSearch() ), actionCollection() );
     _deleteSelected = new KAction( i18n( "Delete Selected" ), QString::fromLatin1("editdelete"), Key_Delete, this, SLOT( slotDeleteSelected() ),
@@ -547,6 +604,8 @@ void MainWindow::Window::setupMenuBar()
                                      actionCollection(), "oneProp" );
     _configAllSimultaniously = new KAction( i18n( "Annotate Multiple Items at a Time" ), CTRL+Key_2, this, SLOT( slotConfigureAllImages() ),
                                             actionCollection(), "allProp" );
+    _rotLeft = new KAction( i18n( "Rotate counterclockwise" ), 0, this, SLOT( slotRotateSelectedLeft() ), actionCollection(), "rotateLeft" );
+    _rotRight = new KAction( i18n( "Rotate clockwise" ), 0, this, SLOT( slotRotateSelectedRight() ), actionCollection(), "rotateRight" );
 
     // The Images menu
     _view = new KAction( i18n("View"), CTRL+Key_I, this, SLOT( slotView() ),
@@ -588,10 +647,13 @@ void MainWindow::Window::setupMenuBar()
     // Maintenance
     new KAction( i18n("Display Images and Videos Not on Disk"), 0, this, SLOT( slotShowNotOnDisk() ), actionCollection(), "findUnavailableImages" );
     new KAction( i18n("Display Images and Videos with Incomplete Dates..."), 0, this, SLOT( slotShowImagesWithInvalidDate() ), actionCollection(), "findImagesWithInvalidDate" );
+    new KAction( i18n("Display Images and Videos with Changed MD5 Sum"), 0, this, SLOT( slotShowImagesWithChangedMD5Sum() ), actionCollection(), "findImagesWithChangedMD5Sum" );
+
     new KAction( i18n("Recalculate Checksum"), 0, this, SLOT( slotRecalcCheckSums() ), actionCollection(), "rebuildMD5s" );
     new KAction( i18n("Rescan for Images and Videos"), 0, DB::ImageDB::instance(), SLOT( slotRescan() ), actionCollection(), "rescan" );
 #ifdef HASEXIV2
-    new KAction( i18n("Read EXIF Info From Files..."), 0, this, SLOT( slotReReadExifInfo() ), actionCollection(), "reReadExifInfo" );
+    new KAction( i18n("Re-Read Metadata from Files..."), 0, this, SLOT( slotReReadExifInfo() ), actionCollection(), "reReadExifInfo" );
+    new KAction( i18n("Write Metadata to Files..."), 0, this, SLOT( slotWriteExifInfo() ), actionCollection(), "writeExifInfo" );
 #endif
 
 #ifdef SQLDB_SUPPORT
@@ -658,13 +720,9 @@ void MainWindow::Window::setupMenuBar()
 
 void MainWindow::Window::slotExportToHTML()
 {
-    QStringList list = selectedOnDisk();
-    if ( list.count() == 0 )
-        list = DB::ImageDB::instance()->currentScope( true );
-
     if ( ! _htmlDialog )
         _htmlDialog = new HTMLGenerator::HTMLDialog( this, "htmlExportDialog" );
-    _htmlDialog->exec( list );
+    _htmlDialog->exec( selectedOnDisk() );
 }
 
 void MainWindow::Window::startAutoSaveTimer()
@@ -853,6 +911,8 @@ void MainWindow::Window::contextMenuEvent( QContextMenuEvent* e )
 #endif
 
         menu.insertSeparator();
+        _rotLeft->plug( &menu );
+        _rotRight->plug( &menu );
         _recreateThumbnails->plug( &menu );
         menu.insertSeparator();
 
@@ -999,6 +1059,47 @@ void MainWindow::Window::slotThumbNailSelectionChanged()
     _configOneAtATime->setEnabled(selection.count() >= 1 );
     _sortByDateAndTime->setEnabled(selection.count() > 1 );
     _recreateThumbnails->setEnabled( selection.count() >= 1 );
+    _rotLeft->setEnabled( selection.count() >= 1 );
+    _rotRight->setEnabled( selection.count() >= 1 );
+}
+
+void MainWindow::Window::rotateSelected( int angle )
+{
+    QStringList list = selected();
+    if ( list.count() == 0 )  {
+        QMessageBox::warning( this,  i18n("No Selection"),  i18n("No item is selected.") );
+    } else {
+        for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it ) {
+            DB::ImageDB::instance()->info( *it )->rotate( angle );
+        }
+        _dirtyIndicator->markDirty();
+        reloadThumbnailsAndFlushCache();
+    }
+}
+
+void MainWindow::Window::slotPluginThatResetsOrientationActivated()
+{
+    QStringList list = selected();
+    if ( list.count() == 0 )  {
+        QMessageBox::warning( this,  i18n("No Selection"),  i18n("Weird, no item is selected... who knows how the plugin action ended...") );
+    } else {
+        for ( QStringList::const_iterator it = list.begin(); it != list.end(); ++it ) {
+            DB::ImageDB::instance()->info( *it )->setAngle( 0 );
+        }
+        _dirtyIndicator->markDirty();
+        reloadThumbnailsAndFlushCache();
+        QMessageBox::information( this, i18n("Orientation changed"), i18n("Rotation information for %1 images has been reset in the database.").arg( list.count() ) );
+    }
+}
+
+void MainWindow::Window::slotRotateSelectedLeft()
+{
+    rotateSelected( -90 );
+}
+
+void MainWindow::Window::slotRotateSelectedRight()
+{
+    rotateSelected( 90 );
 }
 
 void MainWindow::Window::reloadThumbnails(bool flushCache)
@@ -1039,6 +1140,14 @@ void MainWindow::Window::slotShowNotOnDisk()
 }
 
 
+void MainWindow::Window::slotShowImagesWithChangedMD5Sum()
+{
+    Utilities::ShowBusyCursor dummy;
+    StringSet changed = DB::ImageDB::instance()->imagesWithMD5Changed();
+    showThumbNails( changed.toList() );
+}
+
+
 void MainWindow::Window::donateMoney()
 {
     DonateDialog donate( this, "Donate Money" );
@@ -1101,12 +1210,7 @@ void MainWindow::Window::slotImport()
 
 void MainWindow::Window::slotExport()
 {
-    QStringList list = selectedOnDisk();
-    if ( list.count() == 0 ) {
-        KMessageBox::sorry( this, i18n("Nothing to export.") );
-    }
-    else
-        ImportExport::Export::imageExport( list );
+    ImportExport::Export::imageExport( selectedOnDisk() );
 }
 
 void MainWindow::Window::slotReenableMessages()
@@ -1195,6 +1299,29 @@ void MainWindow::Window::plug()
         KActionPtrList actions = plugin->actions();
         for( KActionPtrList::Iterator it = actions.begin(); it != actions.end(); ++it ) {
             KIPI::Category category = plugin->category( *it );
+
+            /**
+             * The following attempt is highly unportable result of three hours
+             * of RTTI fighting. Enjoy :)
+             * 
+             * We can't use
+             * dynamic_cast<KIPIJPEGLossLessPlugin::Plugin_JPEGLossless> because
+             * the header file doesn't have to be available. If we try to use a
+             * lazy 'namespace ... {class ...}' stuff, we get:
+             *
+             * error: cannot dynamic_cast `plugin' (of type `class
+             * KIPI::Plugin*') to type `struct
+             * KIPIJPEGLossLessPlugin::Plugin_JPEGLossless*' (target is not
+             * pointer or reference to complete type).
+             *
+             * So let's just live with this and hope compiler doesn't mangle class
+             * name too much...
+             **/
+            if ( ( QCString( typeid( *plugin ).name() ).find( "Plugin_JPEGLossless" ) != -1 ) && 
+                    ( qstrcmp( (*it)->name(), "rotate_exif" ) == 0 ) ) {
+                connect( *it, SIGNAL( activated() ), this, SLOT( slotPluginThatResetsOrientationActivated() ) );
+            }
+
             if (  category == KIPI::IMAGESPLUGIN ||  category == KIPI::COLLECTIONSPLUGIN )
                 imageActions.append( *it );
 
@@ -1232,6 +1359,7 @@ void MainWindow::Window::slotImagesChanged( const KURL::List& urls )
     for( KURL::List::ConstIterator it = urls.begin(); it != urls.end(); ++it ) {
         ImageManager::ImageLoader::removeThumbnail( (*it).path() );
     }
+    _dirtyIndicator->markDirty();
     reloadThumbnails(true);
 }
 
@@ -1270,6 +1398,7 @@ void MainWindow::Window::slotRemoveTokens()
     if ( !_tokenEditor )
         _tokenEditor = new TokenEditor( this, "token editor" );
     _tokenEditor->show();
+    connect( _tokenEditor, SIGNAL( finished() ), _browser, SLOT( go() ) );
 }
 
 void MainWindow::Window::updateDateBar( const QString& path )
@@ -1367,7 +1496,6 @@ void MainWindow::Window::convertBackend()
             return;
     }
     config->setGroup(QString::fromLatin1("SQLDB"));
-    SQLDB::DatabaseHandler* dbh = 0;
     try {
         SQLDB::DatabaseAddress address = SQLDB::readConnectionParameters(*config);
 
@@ -1394,8 +1522,6 @@ void MainWindow::Window::convertBackend()
     catch (SQLDB::Error& e) {
         KMessageBox::error(this, i18n("Database conversion failed, because following error occurred:\n%1").arg(e.message()));
     }
-    if (dbh)
-        delete dbh;
 #endif
 }
 
@@ -1428,7 +1554,13 @@ void MainWindow::Window::showImage( const QString& fileName )
 
 void MainWindow::Window::slotBuildThumbnails()
 {
-    new ThumbnailView::ThumbnailBuilder( this ); // It will delete itself
+    try {
+        new ThumbnailView::ThumbnailBuilder( this ); // It will delete itself
+    }
+    catch (std::out_of_range&) {
+        // This is thrown if there are no images to show, see ThumbnailBuilder's
+        // documentation.We can safely ignore it here.
+    }
 }
 
 void MainWindow::Window::slotOrderIncr()
