@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2005 Jesper K. Pedersen <blackie@kde.org>
+/* Copyright (C) 2003-2006 Jesper K. Pedersen <blackie@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -19,10 +19,9 @@
 #ifndef CATEGORYIMAGECOLLECTION_H
 #define CATEGORYIMAGECOLLECTION_H
 
-#include <config.h>
 #ifdef HASKIPI
 #include "Plugins/ImageCollection.h"
-#include "imagesearchinfo.h"
+#include "DB/ImageSearchInfo.h"
 #include <kdemacros.h>
 namespace Plugins
 {
@@ -30,11 +29,11 @@ namespace Plugins
 class KDE_EXPORT CategoryImageCollection :public Plugins::ImageCollection {
 
 public:
-    CategoryImageCollection( const ImageSearchInfo& context, const QString& category, const QString& value );
+    CategoryImageCollection( const DB::ImageSearchInfo& context, const QString& category, const QString& value );
     virtual QString name();
     virtual KURL::List images();
 private:
-    ImageSearchInfo _context;
+    DB::ImageSearchInfo _context;
     const QString _category;
     const QString _value;
 };
