@@ -25,7 +25,7 @@
 #include "DB/ImageSearchInfo.h"
 #include "DB/CategoryMatcher.h"
 #include "Utilities/List.h"
-#include <kexidb/transaction.h>
+#include "TransactionGuard.h"
 #include <klocale.h>
 #include <qsize.h>
 
@@ -33,133 +33,6 @@ using namespace SQLDB;
 using Utilities::mergeListsUniqly;
 using Utilities::listSubtract;
 
-QueryHelper::QueryHelper(Connection& connection):
-    _connection(&connection),
-    _driver(connection.driver())
-{
-    if (!_driver)
-        throw InitializationError();
-}
-
-namespace
-{
-    KexiDB::Field::Type fieldTypeFor(QVariant::Type type)
-    {
-        switch (type) {
-        case QVariant::Bool:
-            return KexiDB::Field::Boolean;
-
-        case QVariant::Int:
-        case QVariant::UInt:
-            return KexiDB::Field::Integer;
-
-        case QVariant::LongLong:
-        case QVariant::ULongLong:
-            return KexiDB::Field::BigInteger;
-
-        case QVariant::Double:
-            return KexiDB::Field::Double;
-
-        case QVariant::Date:
-            return KexiDB::Field::Date;
-
-        case QVariant::Time:
-            return KexiDB::Field::Time;
-
-        case QVariant::DateTime:
-            return KexiDB::Field::DateTime;
-
-        case QVariant::String:
-        case QVariant::CString:
-            return KexiDB::Field::Text;
-
-        default:
-            return KexiDB::Field::InvalidType;
-        }
-    }
-}
-
-QString QueryHelper::sqlRepresentation(const QVariant& x) const
-{
-    if (x.type() == QVariant::List) {
-        QStringList repr;
-        QValueList<QVariant> l = x.toList();
-        for (QValueList<QVariant>::const_iterator i = l.begin();
-             i != l.end(); ++i) {
-            repr << sqlRepresentation(*i);
-        }
-        if (repr.isEmpty())
-            return "NULL";
-        else
-            return repr.join(", ");
-    }
-    else
-        // Escape and convert x to string
-        return _driver->valueToSQL(fieldTypeFor(x.type()), x);
-}
-
-void QueryHelper::bindValues(QString &s, const Bindings& b) const
-{
-    int n = 0;
-    for (Bindings::const_iterator i = b.begin(); i != b.end(); ++i) {
-        do {
-            n = s.find("%s", n);
-        } while (n >= 1 && s.at(n - 1) == '%');
-        if (n == -1)
-            break;
-        s = s.replace(n, 2, sqlRepresentation(*i));
-    }
-}
-
-void QueryHelper::executeStatement(const QString& statement,
-                                   const Bindings& bindings)
-{
-    QString s = statement;
-    bindValues(s, bindings);
-
-    //TODO: remove debug
-    qDebug("Executing statement: %s", s.local8Bit().data());
-
-    if (!_connection->executeSQL(s))
-        throw StatementError(_connection->recentSQLString(),
-                             _connection->errorMsg());
-}
-
-QueryResult QueryHelper::executeQuery(const QString& query,
-                                      const Bindings& bindings) const
-{
-    QString q = query;
-    bindValues(q, bindings);
-
-    //TODO: remove debug
-    qDebug("Executing query: %s", q.local8Bit().data());
-
-    KexiDB::Cursor* c = _connection->executeQuery(q);
-    if (!c) {
-        throw QueryError(_connection->recentSQLString(),
-                         _connection->errorMsg());
-    }
-    return QueryResult(c);
-}
-
-Q_ULLONG QueryHelper::insert(const QString& tableName,
-                             const QString& aiFieldName,
-                             const QStringList& fields,
-                             const Bindings& values)
-{
-    Q_ASSERT(fields.count() == values.count());
-
-    QString q = "INSERT INTO %1(%2) VALUES (%3)";
-    q = q.arg(tableName);
-    q = q.arg(fields.join(", "));
-    QStringList l;
-    for (Bindings::size_type i = values.count(); i > 0; --i)
-        l.append("%s");
-    q = q.arg(l.join(", "));
-    executeStatement(q, values);
-    return _connection->lastInsertedAutoIncValue(aiFieldName, tableName);
-}
-
 namespace
 {
 void splitPath(const QString& filename, QString& path, QString& basename)
@@ -203,28 +76,28 @@ QString typeCondition(const QString& fieldName, DB::MediaType typemask)
 
 QStringList QueryHelper::filenames() const
 {
-    QValueList<QString[2]> dirFilePairs =
+    StringStringList dirFilePairs =
         executeQuery("SELECT dir.path, media.filename FROM dir, media "
                      "WHERE media.dirId=dir.id ORDER BY media.place"
-                     ).asString2List();
+                     ).asStringStringList();
     QStringList r;
-    for (QValueList<QString[2]>::const_iterator i = dirFilePairs.begin();
+    for (StringStringList::const_iterator i = dirFilePairs.begin();
          i != dirFilePairs.end(); ++i) {
-        r << makeFullName((*i)[0], (*i)[1]);
+        r << makeFullName((*i).first, (*i).second);
     }
     return r;
 }
 
 QString QueryHelper::mediaItemFilename(int id) const
 {
-    QValueList<QString[2]> dirFilePairs =
+    StringStringList dirFilePairs =
         executeQuery("SELECT dir.path, media.filename FROM dir, media "
-                     "WHERE dir.id=media.dirId AND media.id=%s",
-                     Bindings() << id).asString2List();
+                     "WHERE dir.id=media.dirId AND media.id=?",
+                     Bindings() << id).asStringStringList();
     if (dirFilePairs.isEmpty())
         throw EntryNotFoundError();
 
-    return makeFullName(dirFilePairs[0][0], dirFilePairs[0][1]);
+    return makeFullName(dirFilePairs[0].first, dirFilePairs[0].second);
 }
 
 int QueryHelper::mediaItemId(const QString& filename) const
@@ -235,7 +108,7 @@ int QueryHelper::mediaItemId(const QString& filename) const
     QVariant id =
         executeQuery("SELECT media.id FROM media, dir "
                      "WHERE media.dirId=dir.id AND "
-                     "dir.path=%s AND media.filename=%s",
+                     "dir.path=? AND media.filename=?",
                      Bindings() << path << basename).firstItem();
     if (id.isNull())
         throw EntryNotFoundError();
@@ -277,7 +150,7 @@ QueryHelper::mediaItemIdsForFilenames(const QStringList& filenames) const
         basenames << basename;
         if (!pathIdMap.contains(path)) {
             pathIdMap.insert(path, executeQuery("SELECT id FROM dir "
-                                                "WHERE path=%s",
+                                                "WHERE path=?",
                                                 Bindings() << path
                                                 ).firstItem().toInt());
         }
@@ -287,7 +160,7 @@ QueryHelper::mediaItemIdsForFilenames(const QStringList& filenames) const
     QStringList::const_iterator basenameIt = basenames.begin();
     while (pathIt != paths.end()) {
         QVariant id =
-            executeQuery("SELECT id FROM media WHERE dirId=%s AND filename=%s",
+            executeQuery("SELECT id FROM media WHERE dirId=? AND filename=?",
                          Bindings() << pathIdMap[*pathIt] << *basenameIt
                          ).firstItem();
         /*
@@ -309,7 +182,7 @@ QueryHelper::mediaItemIdsForFilenames(const QStringList& filenames) const
             *i = QString::fromLatin1("./") + *i;
         }
     }
-    executeQuery("SELECT id FROM media, dir WHERE CONCAT(dir.path, '/', media.filename) IN (%s)");
+    executeQuery("SELECT id FROM media, dir WHERE CONCAT(dir.path, '/', media.filename) IN (?)");
 #endif
 }
 
@@ -320,7 +193,7 @@ QStringList QueryHelper::categoryNames() const
 
 int QueryHelper::categoryId(const QString& category) const
 {
-    QVariant r = executeQuery("SELECT id FROM category WHERE name=%s",
+    QVariant r = executeQuery("SELECT id FROM category WHERE name=?",
                               Bindings() << category).firstItem();
     if (r.isNull())
         throw EntryNotFoundError();
@@ -332,7 +205,7 @@ QValueList<int> QueryHelper::tagIdsOfCategory(const QString& category) const
 {
     return executeQuery("SELECT tag.id FROM tag, category "
                         "WHERE tag.categoryId=category.id AND "
-                        "category.name=%s",
+                        "category.name=?",
                         Bindings() << category).asIntegerList();
 }
 
@@ -340,7 +213,7 @@ QStringList QueryHelper::tagNamesOfCategory(int categoryId) const
 {
     // Tags with larger place come first. (NULLs last)
     return executeQuery("SELECT name FROM tag "
-                        "WHERE categoryId=%s ORDER BY place DESC",
+                        "WHERE categoryId=? ORDER BY place DESC",
                         Bindings() << categoryId).asStringList();
 }
 
@@ -367,14 +240,14 @@ uint QueryHelper::mediaItemCount(DB::MediaType typemask,
         else {
             if (typemask == DB::anyMediaType) {
                 return executeQuery("SELECT COUNT(*) FROM media "
-                                    "WHERE id IN (%s)",
+                                    "WHERE id IN (?)",
                                     Bindings() << toVariantList(*scope)
                                     ).firstItem().toUInt();
             }
             else {
                 return executeQuery("SELECT COUNT(*) FROM media WHERE " +
                                     typeCondition("type", typemask) +
-                                    " AND id IN (%s)",
+                                    " AND id IN (?)",
                                     Bindings() << toVariantList(*scope)
                                     ).firstItem().toUInt();
             }
@@ -403,7 +276,7 @@ void QueryHelper::getMediaItem(int id, DB::ImageInfo& info) const
                          "m.startTime, m.endTime, m.width, m.height, m.angle "
                          "FROM media m, dir "
                          "WHERE m.dirId=dir.id AND "
-                         "m.id=%s", Bindings() << id).getRow();
+                         "m.id=?", Bindings() << id).getRow();
     }
     catch (RowNotFoundError&) {
         throw EntryNotFoundError();
@@ -414,7 +287,7 @@ void QueryHelper::getMediaItem(int id, DB::ImageInfo& info) const
     info.delaySavingChanges(true);
 
     info.setFileName(makeFullName(row[0].toString(), row[1].toString()));
-    info.setMD5Sum(row[2].toString());
+    info.setMD5Sum(DB::MD5(row[2].toString()));
     info.setMediaType(static_cast<DB::MediaType>(row[3].toInt()));
     info.setLabel(row[4].toString());
     info.setDescription(row[5].toString());
@@ -426,20 +299,20 @@ void QueryHelper::getMediaItem(int id, DB::ImageInfo& info) const
     info.setSize(QSize(width, height));
     info.setAngle(row[10].toInt());
 
-    QValueList<QString[2]> categoryTagPairs =
+    StringStringList categoryTagPairs =
         executeQuery("SELECT category.name, tag.name "
                      "FROM media_tag, category, tag "
                      "WHERE media_tag.tagId=tag.id AND "
                      "category.id=tag.categoryId AND "
-                     "media_tag.mediaId=%s", Bindings() << id).asString2List();
+                     "media_tag.mediaId=?", Bindings() << id).asStringStringList();
 
-    for (QValueList<QString[2]>::const_iterator i = categoryTagPairs.begin();
+    for (StringStringList::const_iterator i = categoryTagPairs.begin();
          i != categoryTagPairs.end(); ++i)
-        info.addCategoryInfo((*i)[0], (*i)[1]);
+        info.addCategoryInfo((*i).first, (*i).second);
 
     Viewer::DrawList drawList;
     Cursor c = executeQuery("SELECT shape, x0, y0, x1, y1 "
-                            "FROM drawing WHERE mediaId=%s",
+                            "FROM drawing WHERE mediaId=?",
                             Bindings() << id).cursor();
     for (c.selectFirstRow(); c.rowExists(); c.selectNextRow()) {
         RowData row;
@@ -473,13 +346,13 @@ void QueryHelper::getMediaItem(int id, DB::ImageInfo& info) const
 int QueryHelper::insertTag(int categoryId, const QString& name)
 {
     QVariant i = executeQuery("SELECT id FROM tag "
-                              "WHERE categoryId=%s AND name=%s",
+                              "WHERE categoryId=? AND name=?",
                               Bindings() << categoryId << name).firstItem();
     if (!i.isNull())
         return i.toInt();
 
-    return insert("tag", "id", QStringList() << "categoryId" << "name",
-                  Bindings() << categoryId << name);
+    return executeInsert("tag", "id", QStringList() << "categoryId" << "name",
+                         Bindings() << categoryId << name);
 }
 
 void QueryHelper::insertTagFirst(int categoryId, const QString& name)
@@ -487,25 +360,25 @@ void QueryHelper::insertTagFirst(int categoryId, const QString& name)
     // See membersOfCategory() for tag ordering usage.
 
     int id = insertTag(categoryId, name);
-    QVariant oldPlace = executeQuery("SELECT place FROM tag WHERE id=%s",
+    QVariant oldPlace = executeQuery("SELECT place FROM tag WHERE id=?",
                                      Bindings() << id).firstItem();
 
     // Move tags from this to previous first tag one place towards end
     if (!oldPlace.isNull()) {
         executeStatement("UPDATE tag SET place=place-1 "
-                         "WHERE categoryId=%s AND place>%s",
+                         "WHERE categoryId=? AND place>?",
                          Bindings() << categoryId << oldPlace);
     }
 
     // MAX(place) could be NULL, but it'll be returned as 0 with
     // toInt(), which is ok.
     int newPlace = executeQuery("SELECT MAX(place) FROM tag "
-                                "WHERE categoryId=%s",
+                                "WHERE categoryId=?",
                                 Bindings() << categoryId
                                 ).firstItem().toInt() + 1;
 
     // Put this tag first
-    executeStatement("UPDATE tag SET place=%s WHERE id=%s",
+    executeStatement("UPDATE tag SET place=? WHERE id=?",
                      Bindings() << newPlace << id);
 }
 
@@ -513,26 +386,26 @@ void QueryHelper::removeTag(int categoryId, const QString& name)
 {
 #ifndef USEFOREIGNCONSTRAINT
     int tagId =
-        executeQuery("SELECT id FROM tag WHERE categoryId=%s AND name=%s",
+        executeQuery("SELECT id FROM tag WHERE categoryId=? AND name=?",
                      Bindings() << categoryId << name).firstItem().toInt();
-    executeStatement("DELETE FROM media_tag WHERE tagId=%s",
+    executeStatement("DELETE FROM media_tag WHERE tagId=?",
                      Bindings() << tagId);
     executeStatement("DELETE FROM membergroup "
-                     "WHERE groupTag=%s OR memberTag=%s",
+                     "WHERE groupTag=? OR memberTag=?",
                      Bindings() << tagId << tagId);
 #endif
-    executeStatement("DELETE FROM tag WHERE categoryId=%s AND name=%s",
+    executeStatement("DELETE FROM tag WHERE categoryId=? AND name=?",
                      Bindings() << categoryId << name);
 }
 
 void QueryHelper::insertMediaTag(int mediaId, int tagId)
 {
     if (executeQuery("SELECT COUNT(*) FROM media_tag "
-                     "WHERE mediaId=%s AND tagId=%s",
+                     "WHERE mediaId=? AND tagId=?",
                      Bindings() << mediaId << tagId).firstItem().toUInt() > 0)
         return;
     executeStatement("INSERT INTO media_tag (mediaId, tagId) "
-                     "VALUES (%s, %s)", Bindings() << mediaId << tagId);
+                     "VALUES (?, ?)", Bindings() << mediaId << tagId);
 }
 
 void QueryHelper::insertMediaItemTags(int mediaId, const DB::ImageInfo& info)
@@ -576,7 +449,7 @@ void QueryHelper::insertMediaItemDrawings(int mediaId,
 
         executeStatement("INSERT INTO drawing "
                          "(mediaId, shape, x0, y0, x1, y1) "
-                         "VALUES (%s, %s, %s, %s, %s, %s)",
+                         "VALUES (?, ?, ?, ?, ?, ?)",
                          Bindings() << mediaId << shape <<
                          p0.x() << p0.y() << p1.x() << p1.y());
     }
@@ -585,13 +458,13 @@ void QueryHelper::insertMediaItemDrawings(int mediaId,
 int QueryHelper::insertDir(const QString& dirname)
 {
     QVariant i = executeQuery("SELECT id FROM dir "
-                              "WHERE path=%s",
+                              "WHERE path=?",
                               Bindings() << dirname).firstItem();
     if (!i.isNull())
         return i.toInt();
 
-    return insert("dir", "id", QStringList() << "path",
-                  Bindings() << dirname);
+    return executeInsert("dir", "id", QStringList() << "path",
+                         Bindings() << dirname);
 }
 
 /** Get data from ImageInfo to Bindings list.
@@ -607,9 +480,9 @@ QueryHelper::imageInfoToBindings(const DB::ImageInfo& info)
 {
     //Q_ASSERT(bindings.isEmpty());
 
-    QVariant md5 = info.MD5Sum();
-    if (md5.toString().isEmpty())
-        md5 = QVariant();
+    QVariant md5;
+    if (!info.MD5Sum().isNull())
+        md5 = info.MD5Sum().toHexString();
     QVariant w = info.size().width();
     if (w.toInt() == -1)
         w = QVariant();
@@ -635,7 +508,7 @@ void QueryHelper::insertMediaItem(const DB::ImageInfo& info, int place)
     Bindings infoValues = imageInfoToBindings(info);
 
     if (executeQuery("SELECT COUNT(*) FROM media "
-                     "WHERE dirId=%s AND filename=%s",
+                     "WHERE dirId=? AND filename=?",
                      Bindings() << infoValues[0] << infoValues[1]
                      ).firstItem().toUInt() != 0)
         return;
@@ -657,7 +530,7 @@ void QueryHelper::insertMediaItem(const DB::ImageInfo& info, int place)
         "width" << "height" << "angle";
     bindings += infoValues;
 
-    Q_ULLONG mediaId = insert("media", "id", fields, bindings);
+    Q_ULLONG mediaId = executeInsert("media", "id", fields, bindings);
 
     insertMediaItemTags(mediaId, info);
     insertMediaItemDrawings(mediaId, info);
@@ -666,7 +539,7 @@ void QueryHelper::insertMediaItem(const DB::ImageInfo& info, int place)
 void
 QueryHelper::insertMediaItemsLast(const QValueList<DB::ImageInfoPtr>& items)
 {
-    KexiDB::TransactionGuard transaction(*_connection);
+    TransactionGuard transaction(*this);
 
     int place =
         executeQuery("SELECT MAX(place) FROM media").firstItem().toInt() + 1;
@@ -684,24 +557,24 @@ void QueryHelper::updateMediaItem(int id, const DB::ImageInfo& info)
     // TODO: remove debug
     qDebug("Updating info of file %s", info.fileName().local8Bit().data());
 
-    executeStatement("UPDATE media SET dirId=%s, filename=%s, md5sum=%s, "
-                     "type=%s, label=%s, description=%s, "
-                     "startTime=%s, endTime=%s, "
-                     "width=%s, height=%s, angle=%s WHERE id=%s",
+    executeStatement("UPDATE media SET dirId=?, filename=?, md5sum=?, "
+                     "type=?, label=?, description=?, "
+                     "startTime=?, endTime=?, "
+                     "width=?, height=?, angle=? WHERE id=?",
                      imageInfoToBindings(info) << id);
 
-    executeStatement("DELETE FROM media_tag WHERE mediaId=%s",
+    executeStatement("DELETE FROM media_tag WHERE mediaId=?",
                      Bindings() << id);
     insertMediaItemTags(id, info);
 
-    executeStatement("DELETE FROM drawing WHERE mediaId=%s",
+    executeStatement("DELETE FROM drawing WHERE mediaId=?",
                      Bindings() << id);
     insertMediaItemDrawings(id, info);
 }
 
 QValueList<int> QueryHelper::directMembers(int tagId) const
 {
-    return executeQuery("SELECT memberTag FROM membergroup WHERE groupTag=%s",
+    return executeQuery("SELECT memberTag FROM membergroup WHERE groupTag=?",
                         Bindings() << tagId).asIntegerList();
 }
 
@@ -710,7 +583,7 @@ int QueryHelper::tagId(const QString& category, const QString& item) const
     QVariant id =
         executeQuery("SELECT tag.id FROM tag,category "
                      "WHERE tag.categoryId=category.id AND "
-                     "category.name=%s AND tag.name=%s",
+                     "category.name=? AND tag.name=?",
                      Bindings() << category << item).firstItem();
     if (id.isNull())
         throw EntryNotFoundError();
@@ -741,24 +614,15 @@ QValueList<int> QueryHelper::tagIdList(const QString& category,
     return visited;
 }
 
-QValueList<QString[3]> QueryHelper::memberGroupConfiguration() const
-{
-    return executeQuery("SELECT c.name, g.name, m.name "
-                        "FROM membergroup mg, tag g, tag m, category c "
-                        "WHERE mg.groupTag=g.id AND "
-                        "mg.memberTag=m.id AND "
-                        "g.categoryId=c.id").asString3List();
-}
-
-QValueList<QString[2]>
+StringStringList
 QueryHelper::memberGroupConfiguration(const QString& category) const
 {
     return executeQuery("SELECT g.name, m.name "
                         "FROM membergroup mg, tag g, tag m, category c "
                         "WHERE mg.groupTag=g.id AND "
                         "mg.memberTag=m.id AND "
-                        "g.categoryId=c.id AND c.name=%s",
-                        Bindings() << category).asString2List();
+                        "g.categoryId=c.id AND c.name=?",
+                        Bindings() << category).asStringStringList();
 }
 
 void QueryHelper::addBlockItem(const QString& filename)
@@ -768,10 +632,10 @@ void QueryHelper::addBlockItem(const QString& filename)
     splitPath(filename, path, basename);
     int dirId = insertDir(path);
     if (executeQuery("SELECT COUNT(*) FROM blockitem "
-                     "WHERE dirId=%s AND filename=%s", Bindings() <<
+                     "WHERE dirId=? AND filename=?", Bindings() <<
                      dirId << basename).firstItem().asUInt() == 0)
         executeStatement("INSERT INTO blockitem (dirId, filename) "
-                         "VALUES (%s, %s)", Bindings() << dirId << basename);
+                         "VALUES (?, ?)", Bindings() << dirId << basename);
 }
 
 void QueryHelper::addBlockItems(const QStringList& filenames)
@@ -788,7 +652,7 @@ bool QueryHelper::isBlocked(const QString& filename) const
     splitPath(filename, path, basename);
     return executeQuery("SELECT COUNT(*) FROM blockitem, dir "
                         "WHERE blockitem.dirId=dir.id AND "
-                        "dir.path=%s AND blockitem.filename=%s",
+                        "dir.path=? AND blockitem.filename=?",
                         Bindings() << path << basename
                         ).firstItem().toUInt() > 0;
 }
@@ -796,9 +660,9 @@ bool QueryHelper::isBlocked(const QString& filename) const
 void QueryHelper::removeMediaItem(const QString& filename)
 {
     int id = mediaItemId(filename);
-    executeStatement("DELETE FROM media_tag WHERE mediaId=%s",
+    executeStatement("DELETE FROM media_tag WHERE mediaId=?",
                      Bindings() << id);
-    executeStatement("DELETE FROM media WHERE id=%s", Bindings() << id);
+    executeStatement("DELETE FROM media WHERE id=?", Bindings() << id);
 }
 
 void QueryHelper::insertCategory(const QString& name,
@@ -807,7 +671,7 @@ void QueryHelper::insertCategory(const QString& name,
                                  int thumbnailSize)
 {
     executeStatement("INSERT INTO category (name, icon, visible, "
-                     "viewtype, thumbsize) VALUES (%s, %s, %s, %s, %s)",
+                     "viewtype, thumbsize) VALUES (?, ?, ?, ?, ?)",
                      Bindings() << name << icon <<
                      QVariant(visible, 0 /* dummy, to make it bool */) <<
                      type << thumbnailSize);
@@ -816,22 +680,22 @@ void QueryHelper::insertCategory(const QString& name,
 void QueryHelper::removeCategory(const QString& name)
 {
     int id = categoryId(name);
-    Cursor c = executeQuery("SELECT id FROM tag WHERE categoryId=%s").cursor();
+    Cursor c = executeQuery("SELECT id FROM tag WHERE categoryId=?").cursor();
     for (c.selectFirstRow(); c.rowExists(); c.selectNextRow()) {
         QVariant tagId = c.value(0);
-        executeStatement("DELETE FROM media_tag WHERE tagId=%s",
+        executeStatement("DELETE FROM media_tag WHERE tagId=?",
                          Bindings() << tagId);
         executeStatement("DELETE FROM membergroup "
-                         "WHERE groupTag=%s OR memberTag=%s",
+                         "WHERE groupTag=? OR memberTag=?",
                          Bindings() << tagId << tagId);
     }
-    executeStatement("DELETE FROM tag WHERE categoryId=%s", Bindings() << id);
-    executeStatement("DELETE FROM category WHERE id=%s", Bindings() << id);
+    executeStatement("DELETE FROM tag WHERE categoryId=?", Bindings() << id);
+    executeStatement("DELETE FROM category WHERE id=?", Bindings() << id);
 }
 
 QString QueryHelper::categoryName(int id) const
 {
-    QVariant r = executeQuery("SELECT name FROM category WHERE id=%s",
+    QVariant r = executeQuery("SELECT name FROM category WHERE id=?",
                               Bindings() << id).firstItem();
 
     if (r.isNull())
@@ -842,84 +706,101 @@ QString QueryHelper::categoryName(int id) const
 
 QString QueryHelper::categoryIcon(int id) const
 {
-    return executeQuery("SELECT icon FROM category WHERE id=%s",
+    return executeQuery("SELECT icon FROM category WHERE id=?",
                         Bindings() << id).firstItem().toString();
 }
 
 bool QueryHelper::categoryVisible(int id) const
 {
-    return executeQuery("SELECT visible FROM category WHERE id=%s",
+    return executeQuery("SELECT visible FROM category WHERE id=?",
                         Bindings() << id).firstItem().toBool();
 }
 
 DB::Category::ViewType QueryHelper::categoryViewType(int id) const
 {
     return static_cast<DB::Category::ViewType>
-        (executeQuery("SELECT viewtype FROM category WHERE id=%s",
+        (executeQuery("SELECT viewtype FROM category WHERE id=?",
                       Bindings() << id).firstItem().toUInt());
 }
 
 void QueryHelper::changeCategoryName(int id, const QString& newName)
 {
-    executeStatement("UPDATE category SET name=%s WHERE id=%s",
+    executeStatement("UPDATE category SET name=? WHERE id=?",
                      Bindings() << newName << id);
 }
 
 void QueryHelper::changeCategoryIcon(int id, const QString& icon)
 {
-    executeStatement("UPDATE category SET icon=%s WHERE id=%s",
+    executeStatement("UPDATE category SET icon=? WHERE id=?",
                      Bindings() << icon << id);
 }
 
 void QueryHelper::changeCategoryVisible(int id, bool visible)
 {
-    executeStatement("UPDATE category SET visible=%s WHERE id=%s",
+    executeStatement("UPDATE category SET visible=? WHERE id=?",
                      Bindings() << QVariant(visible, 0) << id);
 }
 
 void QueryHelper::changeCategoryViewType(int id, DB::Category::ViewType type)
 {
-    executeStatement("UPDATE category SET viewtype=%s WHERE id=%s",
+    executeStatement("UPDATE category SET viewtype=? WHERE id=?",
                      Bindings() << type << id);
 }
 
-bool QueryHelper::containsMD5Sum(const QString& md5sum) const
+bool QueryHelper::containsMD5Sum(const DB::MD5& md5sum) const
 {
-    return executeQuery("SELECT COUNT(*) FROM media WHERE md5sum=%s",
-                        Bindings() << md5sum).firstItem().toUInt() > 0;
+    return executeQuery("SELECT COUNT(*) FROM media WHERE md5sum=?",
+                        Bindings() << md5sum.toHexString()
+                        ).firstItem().toUInt() > 0;
 }
 
-QString QueryHelper::filenameForMD5Sum(const QString& md5sum) const
+QString QueryHelper::filenameForMD5Sum(const DB::MD5& md5sum) const
 {
-    QValueList<QString[2]> rows =
+    StringStringList rows =
         executeQuery("SELECT dir.path, media.filename FROM dir, media "
-                     "WHERE dir.id=media.dirId AND media.md5sum=%s",
-                     Bindings() << md5sum).asString2List();
+                     "WHERE dir.id=media.dirId AND media.md5sum=?",
+                     Bindings() << md5sum.toHexString()
+                     ).asStringStringList();
     if (rows.isEmpty())
         return QString::null;
     else {
-        return makeFullName(rows[0][0], rows[0][1]);
+        return makeFullName(rows[0].first, rows[0].second);
     }
 }
 
-QValueList< QPair<int, QString> >
-QueryHelper::mediaIdTagPairs(const QString& category,
+QMap<int, StringSet >
+QueryHelper::mediaIdTagsMap(const QString& category,
                              DB::MediaType typemask) const
 {
-    if (category == "Folder")
-        return executeQuery("SELECT media.id, dir.path FROM media, dir "
-                            "WHERE media.dirId=dir.id AND " +
-                            typeCondition("media.type", typemask)
-                            ).asIntegerStringPairs();
-    else
-        return executeQuery("SELECT media.id, tag.name "
-                            "FROM media, media_tag, tag, category "
-                            "WHERE media.id=media_tag.mediaId AND "
-                            "media_tag.tagId=tag.id AND "
-                            "tag.categoryId=category.id AND " +
-                            typeCondition("media.type", typemask) +
-                            " AND category.name=%s",
-                            Bindings() << category).asIntegerStringPairs();
+    QString q;
+    Bindings b;
+
+    if (category == "Folder") {
+        q =
+            "SELECT media.id, dir.path FROM media, dir "
+            "WHERE media.dirId=dir.id AND " +
+            typeCondition("media.type", typemask);
+    }
+    else {
+        q =
+            "SELECT media.id, tag.name "
+            "FROM media, media_tag, tag, category "
+            "WHERE media.id=media_tag.mediaId AND "
+            "media_tag.tagId=tag.id AND "
+            "tag.categoryId=category.id AND " +
+            typeCondition("media.type", typemask) +
+            " AND category.name=?";
+        b.append(category);
+    }
+
+    Cursor c = executeQuery(q, b).cursor();
+
+    QMap<int, StringSet > r;
+    if (!c.isNull()) {
+        for (c.selectFirstRow(); c.rowExists(); c.selectNextRow())
+            r[c.value(0).toInt()].insert(c.value(1).toString());
+    }
+    return r;
 }
 
 int QueryHelper::mediaPlaceByFilename(const QString& filename) const
@@ -930,7 +811,7 @@ int QueryHelper::mediaPlaceByFilename(const QString& filename) const
     QVariant place =
         executeQuery("SELECT media.place FROM media, dir "
                      "WHERE media.dirId=dir.id AND "
-                     "dir.path=%s AND media.filename=%s",
+                     "dir.path=? AND media.filename=?",
                      Bindings() << path << basename).firstItem();
     if (place.isNull())
         throw EntryNotFoundError();
@@ -958,7 +839,7 @@ void QueryHelper::moveMediaItems(const QStringList& filenames,
 
     RowData minmax =
         executeQuery("SELECT MIN(place), MAX(place) "
-                     "FROM media WHERE id IN (%s)",
+                     "FROM media WHERE id IN (?)",
                      Bindings() << toVariantList(srcIds)).getRow();
     int srcMin = minmax[0].toInt();
     int srcMax = minmax[1].toInt();
@@ -967,13 +848,13 @@ void QueryHelper::moveMediaItems(const QStringList& filenames,
     // item in srcIds list.
     QValueList<int> betweenList =
         executeQuery("SELECT id FROM media "
-                     "WHERE %s <= place AND place <= %s AND id NOT IN (%s)",
+                     "WHERE ? <= place AND place <= ? AND id NOT IN (?)",
                      Bindings() << srcMin << srcMax << toVariantList(srcIds)
                      ).asIntegerList();
     int n = srcMin;
     for (QValueList<int>::const_iterator i = betweenList.begin();
          i != betweenList.end(); ++i) {
-        executeStatement("UPDATE media SET place=%s WHERE id=%s",
+        executeStatement("UPDATE media SET place=? WHERE id=?",
                          Bindings() << n << *i);
         ++n;
     }
@@ -982,7 +863,7 @@ void QueryHelper::moveMediaItems(const QStringList& filenames,
     // N, N+1, N+2, ..., N+n.
     for (QValueList<int>::const_iterator i = srcIds.begin();
          i != srcIds.end(); ++i) {
-        executeStatement("UPDATE media SET place=%s WHERE id=%s",
+        executeStatement("UPDATE media SET place=? WHERE id=?",
                          Bindings() << n << *i);
         ++n;
     }
@@ -1000,15 +881,15 @@ void QueryHelper::moveMediaItems(const QStringList& filenames,
     uint N = srcIds.count();
 
     if (low < destPlace)
-        executeStatement("UPDATE media SET place=place-%s "
-                         "WHERE %s <= place AND place <= %s",
+        executeStatement("UPDATE media SET place=place-? "
+                         "WHERE ? <= place AND place <= ?",
                          Bindings() << N << low << destPlace - 1);
     else if (destPlace < high)
-        executeStatement("UPDATE media SET place=place+%s "
-                         "WHERE %s <= place AND place <= %s",
+        executeStatement("UPDATE media SET place=place+? "
+                         "WHERE ? <= place AND place <= ?",
                          Bindings() << N << destPlace << high - 1);
 
-    executeStatement("UPDATE media SET place=place+(%s) WHERE id IN (%s)",
+    executeStatement("UPDATE media SET place=place+(?) WHERE id IN (?)",
                      Bindings() << destPlace - srcMin << toVariantList(srcIds));
 }
 
@@ -1024,7 +905,7 @@ void QueryHelper::makeMediaPlacesContinuous()
 
 void QueryHelper::sortMediaItems(const QStringList& filenames)
 {
-    KexiDB::TransactionGuard transaction(*_connection);
+    TransactionGuard transaction(*this);
 
     QValueList<int> idList = mediaItemIdsForFilenames(filenames);
 
@@ -1034,11 +915,11 @@ void QueryHelper::sortMediaItems(const QStringList& filenames)
     executeStatement("UPDATE media, "
                      "(SELECT a.place AS place, COUNT(*) AS n "
                      " FROM media a, media b "
-                     " WHERE a.id IN (%s) AND b.id IN (%s) AND "
+                     " WHERE a.id IN (?) AND b.id IN (?) AND "
                      " b.place <= a.place GROUP BY a.id) byplace, "
                      "(SELECT a.id AS id, COUNT(*) AS n "
                      " FROM media a, media b "
-                     " WHERE a.id IN (%s) AND b.id IN (%s) AND "
+                     " WHERE a.id IN (?) AND b.id IN (?) AND "
                      " b.startTime <= a.startTime GROUP BY a.id) bytime "
                      "SET media.place=byplace.place "
                      "WHERE media.id=bytime.id AND byplace.n=bytime.n",
@@ -1046,7 +927,7 @@ void QueryHelper::sortMediaItems(const QStringList& filenames)
 #else
     executeStatement("CREATE TEMPORARY TABLE sorttmp "
                      "SELECT id, place, startTime "
-                     "FROM media WHERE id IN (%s)",
+                     "FROM media WHERE id IN (?)",
                      Bindings() << toVariantList(idList));
 
     idList =
@@ -1059,7 +940,7 @@ void QueryHelper::sortMediaItems(const QStringList& filenames)
     QValueList<int>::const_iterator idIt = idList.begin();
     QValueList<int>::const_iterator placeIt = placeList.begin();
     for (; idIt != idList.end(); ++idIt, ++placeIt) {
-        executeStatement("UPDATE sorttmp SET place=%s WHERE id=%s",
+        executeStatement("UPDATE sorttmp SET place=? WHERE id=?",
                          Bindings() << *placeIt << *idIt);
 
     }
@@ -1098,27 +979,28 @@ QueryHelper::findFirstFileInTimeRange(const DB::ImageDate& range,
     Bindings bindings;
 
     if (idList) {
-        query += "media.id IN (%s) AND ";
+        query += "media.id IN (?) AND ";
         bindings << toVariantList(*idList);
     }
 
     if (!includeRanges) {
-        query += "%s <= media.startTime AND media.endTime <= %s";
+        query += "? <= media.startTime AND media.endTime <= ?";
     }
     else {
-        query += "%s <= media.endTime AND media.startTime <= %s";
+        query += "? <= media.endTime AND media.startTime <= ?";
     }
     bindings << range.start() << range.end();
 
     query += " ORDER BY media.startTime LIMIT 1";
 
-    QValueList<QString[2]> dirFilenamePairs =
-        executeQuery(query, bindings).asString2List();
+    StringStringList dirFilenamePairs =
+        executeQuery(query, bindings).asStringStringList();
 
     if (dirFilenamePairs.isEmpty())
         return QString::null;
     else {
-        return makeFullName(dirFilenamePairs[0][0], dirFilenamePairs[0][1]);
+        return makeFullName(dirFilenamePairs[0].first,
+                            dirFilenamePairs[0].second);
     }
 }
 
@@ -1190,13 +1072,13 @@ QueryHelper::getMatchingFiles(MatcherList matches,
         if (m->_category == "Folder") {
             positiveQuery <<
                 "id IN (SELECT media.id FROM media, dir "
-                "WHERE media.dirId=dir.id AND dir.path=%s)";
+                "WHERE media.dirId=dir.id AND dir.path=?)";
             binds << m->_option;
             matchedFolders += m->_option;
         }
         else {
             positiveQuery <<
-                "id IN (SELECT mediaId FROM media_tag WHERE tagId IN (%s))";
+                "id IN (SELECT mediaId FROM media_tag WHERE tagId IN (?))";
             QValueList<int> tagIds = tagIdList(m->_category, m->_option);
             binds << toVariantList(tagIds);
             matchedTags[m->_category] += tagIds;
@@ -1214,13 +1096,13 @@ QueryHelper::getMatchingFiles(MatcherList matches,
             if (m->_category == "Folder") {
                 negativeQuery <<
                     "id NOT IN (SELECT media.id FROM media, dir "
-                    "WHERE media.dirId=dir.id AND dir.path=%s)";
+                    "WHERE media.dirId=dir.id AND dir.path=?)";
                 binds << m->_option;
             }
             else {
                 negativeQuery <<
                     "id NOT IN (SELECT mediaId "
-                    "FROM media_tag WHERE tagId IN (%s))";
+                    "FROM media_tag WHERE tagId IN (?))";
                 binds << toVariantList(tagIdList(m->_category, m->_option));
             }
         }
@@ -1237,7 +1119,7 @@ QueryHelper::getMatchingFiles(MatcherList matches,
                 if (!excludedFolders.isEmpty()) {
                     excludeQuery <<
                         "id IN (SELECT media.id FROM media, dir "
-                        "WHERE media.dirId=dir.id AND dir.path IN (%s))";
+                        "WHERE media.dirId=dir.id AND dir.path IN (?))";
                     excBinds << toVariantList(excludedFolders);
                 }
             }
@@ -1251,7 +1133,7 @@ QueryHelper::getMatchingFiles(MatcherList matches,
                 if (!excludedTags.isEmpty()) {
                     excludeQuery <<
                         "id IN (SELECT mediaId "
-                        "FROM media_tag WHERE tagId IN (%s))";
+                        "FROM media_tag WHERE tagId IN (?))";
                     excBinds << toVariantList(excludedTags);
                 }
             }
@@ -1297,16 +1179,16 @@ QueryHelper::classify(const QString& category,
     QMap<QString, uint> result;
     DB::GroupCounter counter( category );
 
-    QValueList< QPair<int, QString> > idTagPairs =
-        mediaIdTagPairs(category, typemask);
+    const QMap<int, StringSet > wholeItemMap =
+        mediaIdTagsMap(category, typemask);
 
     QMap<int,QStringList> itemMap;
-    for (QValueList< QPair<int, QString> >::const_iterator
-             i = idTagPairs.begin(); i != idTagPairs.end(); ++i) {
-        int fileId = (*i).first;
-        QString item = (*i).second;
+
+    for (QMap<int, StringSet >::const_iterator
+             i = wholeItemMap.begin(); i != wholeItemMap.end(); ++i) {
+        int fileId = i.key();
         if (!scope || scope->contains(fileId))
-            itemMap[fileId].append(item);
+            itemMap[fileId] = i.data().toList();
     }
 
     // Count images that doesn't contain an item
