@@ -28,18 +28,16 @@
 
 namespace SQLDB
 {
+    typedef QValueList< QPair<QString, QString> > StringStringList;
+
     class QueryResult
     {
     public:
         QueryResult(KexiDB::Cursor* cursor): _cursor(cursor) {}
 
-        QStringList asStringList() const;
-        QValueList<QString[2]> asString2List() const;
-        QValueList<QString[3]> asString3List() const;
         QValueList<int> asIntegerList() const;
-        QValueList< QPair<int, int> > asInteger2List() const;
-        QValueList< QPair<int, QString> > asIntegerStringPairs() const;
-        QMap<int, QString> asIntegerStringMap() const;
+        QStringList asStringList() const;
+        StringStringList asStringStringList() const;
         QVariant firstItem() const;
         RowData getRow(uint n=0) const;
         Cursor cursor() const { return _cursor; }
