@@ -19,7 +19,6 @@
 #ifndef SQLIMAGEDATERANGECOLLECTION_H
 #define SQLIMAGEDATERANGECOLLECTION_H
 #include "DB/ImageDateCollection.h"
-#include "Connection.h"
 #include "QueryHelper.h"
 
 namespace SQLDB
@@ -28,14 +27,14 @@ namespace SQLDB
 class SQLImageDateCollection :public DB::ImageDateCollection
 {
 public:
-    explicit SQLImageDateCollection(Connection& connection);
+    explicit SQLImageDateCollection(QueryHelper& queryHelper);
 
     virtual DB::ImageCount count( const DB::ImageDate& date );
     virtual QDateTime lowerLimit() const;
     virtual QDateTime upperLimit() const;
 
 protected:
-    QueryHelper _qh;
+    QueryHelper& _qh;
 };
 
 }
