@@ -22,21 +22,21 @@
 
 using namespace SQLDB;
 
-SQLMD5Map::SQLMD5Map(Connection& connection):
-    _qh(connection)
+SQLMD5Map::SQLMD5Map(QueryHelper& queryHelper):
+    _qh(queryHelper)
 {
 }
 
-void SQLMD5Map::insert(const QString&, const QString&)
+void SQLMD5Map::insert(const DB::MD5&, const QString&)
 {
 }
 
-QString SQLMD5Map::lookup(const QString& md5sum)
+QString SQLMD5Map::lookup(const DB::MD5& md5sum)
 {
     return _qh.filenameForMD5Sum(md5sum);
 }
 
-bool SQLMD5Map::contains(const QString& md5sum)
+bool SQLMD5Map::contains(const DB::MD5& md5sum)
 {
     return _qh.containsMD5Sum(md5sum);
 }
