@@ -21,7 +21,6 @@
 #define SQLMD5MAP_H
 
 #include "DB/MD5Map.h"
-#include "Connection.h"
 #include "QueryHelper.h"
 
 namespace SQLDB
@@ -29,15 +28,15 @@ namespace SQLDB
     class SQLMD5Map : public DB::MD5Map
     {
     public:
-        explicit SQLMD5Map(Connection& connection);
+        explicit SQLMD5Map(QueryHelper& queryHelper);
 
-        void insert(const QString& md5sum, const QString& fileName);
-        QString lookup(const QString& md5sum);
-        bool contains(const QString& md5sum);
+        void insert(const DB::MD5& md5sum, const QString& fileName);
+        QString lookup(const DB::MD5& md5sum);
+        bool contains(const DB::MD5& md5sum);
         void clear();
 
     protected:
-        QueryHelper _qh;
+        QueryHelper& _qh;
     };
 }
 
