@@ -36,8 +36,12 @@
 #include <qwhatsthis.h>
 #include <kglobal.h>
 #include <kiconloader.h>
-#include <qvgroupbox.h>
+#include <qhgroupbox.h>
 #include <qhbox.h>
+#include <qvbox.h>
+#include <qgrid.h>
+#include <qvbuttongroup.h>
+#include <qtabwidget.h>
 #include "ViewerSizeConfig.h"
 #include <limits.h>
 #include <config.h>
@@ -53,6 +57,7 @@
 #ifdef HASEXIV2
 #  include "Exif/Info.h"
 #  include "Exif/TreeView.h"
+#  include "Exif/SyncWidget.h"
 #endif
 
 #ifdef SQLDB_SUPPORT
@@ -72,6 +77,7 @@ Settings::SettingsDialog::SettingsDialog( QWidget* parent, const char* name )
     createViewerPage();
     createPluginPage();
     createEXIFPage();
+    createSyncPage();
     createDatabaseBackendPage();
 
     connect( this, SIGNAL( aboutToShowPage( QWidget* ) ), this, SLOT( slotPageChange() ) );
@@ -91,19 +97,9 @@ void Settings::SettingsDialog::createGeneralPage()
 
     // Thrust time stamps
     QWidget* container = new QWidget( box );
-    QLabel* timeStampLabel = new QLabel( i18n("Trust image dates:"), container );
-    _trustTimeStamps = new KComboBox( container );
-    _trustTimeStamps->insertStringList( QStringList() << i18n("Always") << i18n("Ask") << i18n("Never") );
     QHBoxLayout* hlay = new QHBoxLayout( container, 0, 6 );
-    hlay->addWidget( timeStampLabel );
-    hlay->addWidget( _trustTimeStamps );
     hlay->addStretch( 1 );
 
-    // Do EXIF rotate
-    _useEXIFRotate = new QCheckBox( i18n( "Use EXIF orientation information" ), box );
-
-    _useEXIFComments = new QCheckBox( i18n( "Use EXIF description" ), box );
-
     // Search for images on startup
     _searchForImagesOnStartup = new QCheckBox( i18n("Search for new images and videos on startup"), box );
     _dontReadRawFilesWithOtherMatchingFile = new QCheckBox( i18n("Don't read RAW files if matching JPEG/TIFF file exists"), box );
@@ -145,28 +141,6 @@ void Settings::SettingsDialog::createGeneralPage()
     // Whats This
     QString txt;
 
-    txt = i18n( "<p>KPhotoAlbum will try to read the image date from EXIF information in the image. "
-                "If that fails it will try to get the date from the file's time stamp.</p>"
-                "<p>However, this information will be wrong if the image was scanned in (you want the date the image "
-                "was taken, not the date of the scan).</p>"
-                "<p>If you only scan images, in contrast to sometimes using "
-                "a digital camera, you should reply <b>no</b>. If you never scan images, you should reply <b>yes</b>, "
-                "otherwise reply <b>ask</b>. This will allow you to decide whether the images are from "
-                "the scanner or the camera, from session to session.</p>" );
-    QWhatsThis::add( timeStampLabel, txt );
-    QWhatsThis::add( _trustTimeStamps, txt );
-
-    txt = i18n( "<p>JPEG images may contain information about rotation. "
-                "If you have a reason for not using this information to get a default rotation of "
-                "your images, uncheck this check box.</p>"
-                "<p>Note: Your digital camera may not write this information into the images at all.</p>" );
-    QWhatsThis::add( _useEXIFRotate, txt );
-
-    txt = i18n( "<p>JPEG images may contain a description. "
-               "Check this checkbox to specify if you want to use this as a "
-               "default description for your images.</p>" );
-    QWhatsThis::add( _useEXIFComments, txt );
-
     txt = i18n( "<p>KPhotoAlbum is capable of searching for new images and videos when started, this does, "
                 "however, take some time, so instead you may wish to manually tell KPhotoAlbum to search for new images "
                 "using <b>Maintenance->Rescan for new images</b></p>");
@@ -222,6 +196,33 @@ void Settings::SettingsDialog::createThumbNailPage()
     lay->addWidget( thumbnailSizeLabel, row, 0 );
     lay->addWidget( _thumbnailSize, row, 1 );
 
+    // Thumbnail aspect ratio
+    ++row;
+    QLabel* thumbnailAspectRatioLabel = new QLabel( i18n("Thumbnail table cells aspect ratio"), top, "thumbnailAspectRatioLabel");
+    _thumbnailAspectRatio = new KComboBox( top );
+    _thumbnailAspectRatio->insertStringList( QStringList() << i18n("1:1") << i18n("4:3")
+        << i18n("3:2") << i18n("16:9") << i18n("3:4") << i18n("2:3") << i18n("9:16"));
+    lay->addWidget( thumbnailAspectRatioLabel, row, 0 );
+    lay->addWidget( _thumbnailAspectRatio, row, 1 );
+
+    // Space around cells
+    ++row;
+    QLabel* thumbnailSpaceLabel = new QLabel( i18n("Space around cells"), top, "thumbnailSpaceLabel");
+    _thumbnailSpace = new QSpinBox( 0, 20, 1, top );
+    lay->addWidget( thumbnailSpaceLabel, row, 0 );
+    lay->addWidget( _thumbnailSpace, row, 1 );
+
+    // Display dark background
+    ++row;
+    _thumbnailDarkBackground = new QCheckBox( i18n("Show thumbnails on dark background" ), top, "thumbnailDarkBackground");
+    lay->addMultiCellWidget( _thumbnailDarkBackground, row, row, 0, 1 );
+
+    // Display grid lines in the thumbnail view
+    ++row;
+    _thumbnailDisplayGrid = new QCheckBox( i18n("Display grid around thumbnails" ),
+                                           top, "_thumbnailDisplayGrid");
+    lay->addMultiCellWidget( _thumbnailDisplayGrid, row, row, 0, 1 );
+
     // Display Labels
     ++row;
     _displayLabels = new QCheckBox( i18n("Display labels in thumbnail view" ), top, "displayLabels" );
@@ -243,7 +244,7 @@ void Settings::SettingsDialog::createThumbNailPage()
     // Thumbnail Cache
     ++row;
     QLabel* cacheLabel = new QLabel( i18n( "Thumbnail cache:" ), top );
-    _thumbnailCache = new QSpinBox( 1, 256, 1, top );
+    _thumbnailCache = new QSpinBox( 1, 4096, 1, top );
     _thumbnailCache->setSuffix( i18n("Mbytes" ) );
     lay->addWidget( cacheLabel, row, 0 );
     lay->addWidget( _thumbnailCache, row, 1 );
@@ -265,6 +266,20 @@ void Settings::SettingsDialog::createThumbNailPage()
     QWhatsThis::add( thumbnailSizeLabel, txt );
     QWhatsThis::add( _thumbnailSize, txt );
 
+    txt = i18n("<p>Choose what aspect ratio the cells holding thumbnails should have.</p>");
+    QWhatsThis::add( _thumbnailAspectRatio, txt );
+
+    txt = i18n("<p>How thick the cell padding should be.</p>");
+    QWhatsThis::add( thumbnailSpaceLabel, txt );
+
+    txt = i18n("<p>The background color in the thumbnail view is "
+               "the systems' default background; by checking this option, "
+               "you can set it to be dark instead.</p>");
+    QWhatsThis::add( _thumbnailDarkBackground, txt );
+
+    txt = i18n("<p>If you want to see grid around your thumbnail images, "
+               "select this option.</p>");
+    QWhatsThis::add( _thumbnailDisplayGrid, txt );
 
     txt = i18n("<p>Checking this option will show the base name for the file under "
                "thumbnails in the thumbnail view.</p>");
@@ -366,9 +381,6 @@ void Settings::SettingsDialog::show()
     // General page
     _previewSize->setValue( opt->previewSize() );
     _thumbnailSize->setValue( opt->thumbSize() );
-    _trustTimeStamps->setCurrentItem( opt->tTimeStamps() );
-    _useEXIFRotate->setChecked( opt->useEXIFRotate() );
-    _useEXIFComments->setChecked( opt->useEXIFComments() );
     _searchForImagesOnStartup->setChecked( opt->searchForImagesOnStartup() );
     _dontReadRawFilesWithOtherMatchingFile->setChecked( opt->dontReadRawFilesWithOtherMatchingFile() );
     _compressedIndexXML->setChecked( opt->useCompressedIndexXML() );
@@ -384,6 +396,10 @@ void Settings::SettingsDialog::show()
         cat = DB::ImageDB::instance()->categoryCollection()->categories()[0];
     _albumCategory->setCurrentText( cat->text() );
 
+    _thumbnailDarkBackground->setChecked( opt->thumbnailDarkBackground() );
+    _thumbnailDisplayGrid->setChecked( opt->thumbnailDisplayGrid() );
+    _thumbnailAspectRatio->setCurrentItem( opt->thumbnailAspectRatio() );
+    _thumbnailSpace->setValue( opt->thumbnailSpace() );
     _displayLabels->setChecked( opt->displayLabels() );
     _displayCategories->setChecked( opt->displayCategories() );
     _viewImageSetup->setSize( opt->viewerSize() );
@@ -406,7 +422,10 @@ void Settings::SettingsDialog::show()
     QValueList<DB::CategoryPtr> categories = DB::ImageDB::instance()->categoryCollection()->categories();
     for( QValueList<DB::CategoryPtr>::Iterator it = categories.begin(); it != categories.end(); ++it ) {
         if( !(*it)->isSpecialCategory() ) {
-            new CategoryItem( (*it)->name(), (*it)->text(),(*it)->iconName(),(*it)->viewType(), (*it)->thumbnailSize(), _categories );
+            CategoryItem* item = new CategoryItem( (*it)->name(), (*it)->text(),(*it)->iconName(),(*it)->viewType(), (*it)->thumbnailSize(), _categories );
+            connect( item, SIGNAL( categoryRenamed( const QString&, const QString& ) ), this, SLOT( slotCategoryRenamed( const QString&, const QString& ) ) );
+            connect( item, SIGNAL( categoryAdded( const QString& ) ), this, SLOT( slotCategoryAdded( const QString& ) ) );
+            connect( item, SIGNAL( categoryRemoved( const QString& ) ), this, SLOT( slotCategoryRemoved( const QString& ) ) );
         }
     }
 
@@ -415,6 +434,29 @@ void Settings::SettingsDialog::show()
     _exifForDialog->reload();
     _exifForViewer->setSelected( Settings::SettingsData::instance()->exifForViewer() );
     _exifForDialog->setSelected( Settings::SettingsData::instance()->exifForDialog() );
+    _iptcCharset->setCurrentItem( opt->iptcCharset() );
+
+    // Synchronization page
+    _orientationRead->updatePreferred( Settings::SettingsData::instance()->orientationSyncing( false ) );
+    _orientationWrite->updatePreferred( Settings::SettingsData::instance()->orientationSyncing( true ) );
+    _labelRead->updatePreferred( Settings::SettingsData::instance()->labelSyncing( false ) );
+    _labelWrite->updatePreferred( Settings::SettingsData::instance()->labelSyncing( true ) );
+    _descriptionRead->updatePreferred( Settings::SettingsData::instance()->descriptionSyncing( false ) );
+    _descriptionWrite->updatePreferred( Settings::SettingsData::instance()->descriptionSyncing( true ) );
+    _dateRead->updatePreferred( Settings::SettingsData::instance()->dateSyncing( false ) );
+    _dateWrite->updatePreferred( Settings::SettingsData::instance()->dateSyncing( true ) );
+    for( QValueList<DB::CategoryPtr>::Iterator it = categories.begin(); it != categories.end(); ++it )
+        if( !(*it)->isSpecialCategory() ) {
+            if ( !_catFieldsRead[ (*it)->name() ] ) {
+                qDebug("No record for category \"%s\"", (*it)->name().ascii() );
+                continue;
+            }
+            _catFieldsRead[ (*it)->name() ]->updatePreferred( Settings::SettingsData::instance()->categorySyncingFields( false, (*it)->name() ) );
+            _catFieldsWrite[ (*it)->name() ]->updatePreferred( Settings::SettingsData::instance()->categorySyncingFields( true, (*it)->name() ) );
+            _catSuper[ (*it)->name() ]->setCurrentItem( opt->categorySyncingSuperGroups( (*it)->name() ) );
+            _catMulti[ (*it)->name() ]->setCurrentItem( opt->categorySyncingMultiValue( (*it)->name() ) );
+            _catAddName[ (*it)->name() ]->setChecked( opt->categorySyncingAddName( (*it)->name() ) );
+        }
 #endif
 
     QString backend = Settings::SettingsData::instance()->backend();
@@ -448,9 +490,7 @@ void Settings::SettingsDialog::slotMyOK()
 
     opt->setPreviewSize( _previewSize->value() );
     opt->setThumbSize( _thumbnailSize->value() );
-    opt->setTTimeStamps( (TimeStampTrust) _trustTimeStamps->currentItem() );
-    opt->setUseEXIFRotate( _useEXIFRotate->isChecked() );
-    opt->setUseEXIFComments( _useEXIFComments->isChecked() );
+    opt->setThumbnailAspectRatio( (ThumbnailAspectRatio) _thumbnailAspectRatio->currentItem() );
     opt->setSearchForImagesOnStartup( _searchForImagesOnStartup->isChecked() );
     opt->setDontReadRawFilesWithOtherMatchingFile( _dontReadRawFilesWithOtherMatchingFile->isChecked() );
     opt->setBackupCount( _backupCount->value() );
@@ -464,6 +504,9 @@ void Settings::SettingsDialog::slotMyOK()
     opt->setHistogramSize( QSize( _barWidth->value(), _barHeight->value() ) );
 
     opt->setAlbumCategory( name );
+    opt->setThumbnailDarkBackground( _thumbnailDarkBackground->isChecked() );
+    opt->setThumbnailDisplayGrid( _thumbnailDisplayGrid->isChecked() );
+    opt->setThumbnailSpace( _thumbnailSpace->value() );
     opt->setDisplayLabels( _displayLabels->isChecked() );
     opt->setDisplayCategories( _displayCategories->isChecked() );
     opt->setViewerSize( _viewImageSetup->size() );
@@ -504,6 +547,33 @@ void Settings::SettingsDialog::slotMyOK()
 #ifdef HASEXIV2
     opt->setExifForViewer( _exifForViewer->selected() ) ;
     opt->setExifForDialog( _exifForDialog->selected() ) ;
+    opt->setIptcCharset( _iptcCharset->currentItem() );
+
+    // Synchronization
+    opt->setOrientationSyncing( false, _orientationRead->items() );
+    opt->setOrientationSyncing( true, _orientationWrite->items() );
+    opt->setLabelSyncing( false, _labelRead->items() );
+    opt->setLabelSyncing( true, _labelWrite->items() );
+    opt->setDescriptionSyncing( false, _descriptionRead->items() );
+    opt->setDescriptionSyncing( true, _descriptionWrite->items() );
+    opt->setDateSyncing( false, _dateRead->items() );
+    opt->setDateSyncing( true, _dateWrite->items() );
+
+    for (QDictIterator<Exif::SyncWidget> it( _catFieldsRead ); it.current(); ++it ) {
+        opt->setCategorySyncingFields( false, it.currentKey(), it.current()->items() );
+    }
+    for (QDictIterator<Exif::SyncWidget> it( _catFieldsWrite ); it.current(); ++it ) {
+        opt->setCategorySyncingFields( true, it.currentKey(), it.current()->items() );
+    }
+    for (QDictIterator<KComboBox> it( _catSuper ); it.current(); ++it ) {
+        opt->setCategorySyncingSuperGroups( it.currentKey(), static_cast<Exif::Syncable::SuperGroupHandling>( it.current()->currentItem() ) );
+    }
+    for (QDictIterator<KComboBox> it( _catMulti ); it.current(); ++it ) {
+        opt->setCategorySyncingMultiValue( it.currentKey(), static_cast<Exif::Syncable::MultiValueHandling>( it.current()->currentItem() ) );
+    }
+    for (QDictIterator<QCheckBox> it( _catAddName ); it.current(); ++it ) {
+        opt->setCategorySyncingAddName( it.currentKey(), it.current()->isChecked() );
+    }
 #endif
 
     // SQLDB
@@ -563,8 +633,17 @@ void Settings::SettingsDialog::slotIconChanged( QString icon )
 
 void Settings::SettingsDialog::slotNewItem()
 {
-    _current = new CategoryItem( QString::null, QString::null, QString::null, DB::Category::ListView, 64, _categories );
-    _text->setText( QString::fromLatin1( "" ) );
+    bool ok;
+    QString name = KInputDialog::getText( i18n("New category"), i18n("Name"), QString::null, &ok );
+    if ( !ok )
+        return;
+
+    _current = new CategoryItem( QString::null, name, QString::null, DB::Category::ListView, 64, _categories );
+    connect( _current, SIGNAL( categoryRenamed( const QString&, const QString& ) ), this, SLOT( slotCategoryRenamed( const QString&, const QString& ) ) );
+    connect( _current, SIGNAL( categoryAdded( const QString& ) ), this, SLOT( slotCategoryAdded( const QString& ) ) );
+    connect( _current, SIGNAL( categoryRemoved( const QString& ) ), this, SLOT( slotCategoryRemoved( const QString& ) ) );
+
+    _text->setText( name );
     _icon->setIcon( QString::null );
     _thumbnailSizeInCategory->setValue( 64 );
     enableDisable( true );
@@ -578,6 +657,7 @@ void Settings::SettingsDialog::slotDeleteCurrent()
     if ( answer == KMessageBox::No )
         return;
 
+    //slotCategoryRemoved( _current->text() );
     _deleted.append( _current );
     _categories->takeItem( _current );
     _current = 0;
@@ -856,7 +936,7 @@ void Settings::SettingsDialog::createViewerPage()
     label = new QLabel( i18n("Image cache:"), top );
     glay->addWidget( label, 1, 0 );
 
-    _cacheSize = new QSpinBox( 0, 2000, 10, top, "_cacheSize" );
+    _cacheSize = new QSpinBox( 0, 4096, 10, top, "_cacheSize" );
     _cacheSize->setSuffix( i18n(" Mbytes") );
     glay->addWidget( _cacheSize, 1, 1 );
     label->setBuddy( _cacheSize );
@@ -914,19 +994,153 @@ void Settings::SettingsDialog::createPluginPage()
 void Settings::SettingsDialog::createEXIFPage()
 {
 #ifdef HASEXIV2
-    QWidget* top = addPage( i18n("EXIF Information" ), i18n("EXIF Information" ),
+    QWidget* top = addPage( i18n("EXIF View" ), i18n("EXIF/IPTC Information" ),
                             KGlobal::iconLoader()->loadIcon( QString::fromLatin1( "contents" ),
                                                              KIcon::Desktop, 32 ) );
-    QHBoxLayout* lay1 = new QHBoxLayout( top, 6 );
+    QVBoxLayout* vlay = new QVBoxLayout( top );
+    QHBoxLayout* hlay1 = new QHBoxLayout( vlay );
+    QHBoxLayout* hlay2 = new QHBoxLayout( vlay );
+    hlay1->setSpacing( 6 );
+    vlay->setSpacing( 6 );
+    hlay2->setSpacing( 6 );
 
-    _exifForViewer = new Exif::TreeView( i18n( "EXIF info to show in the Viewer" ), top );
-    lay1->addWidget( _exifForViewer );
+    _exifForViewer = new Exif::TreeView( i18n( "EXIF/IPTC info to show in the Viewer" ), top );
+    hlay1->addWidget( _exifForViewer );
+
+    _exifForDialog = new Exif::TreeView( i18n("EXIF/IPTC info to show in the EXIF dialog"), top );
+    hlay1->addWidget( _exifForDialog );
+
+    QLabel* _iptcCharsetLabel = new QLabel( i18n("Character set for image metadata:"), top, "iptcCharsetLabel" );
+    _iptcCharset = new KComboBox( top );
+    _iptcCharset->insertStringList( Utilities::humanReadableCharsetList() );
+    QWhatsThis::add( _iptcCharset, i18n("<p>Which character set to use for reading/writing of EXIF and IPTC data</p>") );
+
+    hlay2->addStretch( 1 );
+    hlay2->addWidget( _iptcCharsetLabel );
+    hlay2->addWidget( _iptcCharset );
 
-    _exifForDialog = new Exif::TreeView( i18n("EXIF info to show in the EXIF dialog"), top );
-    lay1->addWidget( _exifForDialog );
 #endif
 }
 
+void Settings::SettingsDialog::createSyncPage()
+{
+#ifdef HASEXIV2
+    _syncPage = addPage( i18n("Metadata"), i18n("Metadata Synchronization"),
+                        KGlobal::iconLoader()->loadIcon( QString::fromLatin1( "saveas" ),
+                                                         KIcon::Desktop, 32 ) );
+
+    const Settings::SettingsData* opt = Settings::SettingsData::instance();
+
+    QVBoxLayout* _lay = new QVBoxLayout( _syncPage );
+    _syncTabs = new QTabWidget( _syncPage );
+    _syncTabs->setMargin( 6 );
+    QHBox* hbox;
+
+    hbox = new QHBox( _syncPage );
+    hbox->setSpacing( 6 );
+    _orientationRead = new Exif::SyncWidget( i18n("Fields to get value from"), hbox,
+            opt->defaultFields( false, QString::fromAscii("orientation")) );
+    _orientationWrite = new Exif::SyncWidget( i18n("Fields to write value to"), hbox,
+            opt->defaultFields( true, QString::fromAscii("orientation")) );
+    _syncTabs->addTab( hbox, i18n("Image orientation") );
+
+    hbox = new QHBox( _syncPage );
+    hbox->setSpacing( 6 );
+    _labelRead = new Exif::SyncWidget( i18n("Fields to get value from"), hbox,
+            opt->defaultFields( false, QString::fromAscii("label")) );
+    _labelWrite = new Exif::SyncWidget( i18n("Fields to write value to"), hbox,
+            opt->defaultFields( true, QString::fromAscii("label")) );
+    _syncTabs->addTab( hbox, i18n("Label") );
+
+    hbox = new QHBox( _syncPage );
+    hbox->setSpacing( 6 );
+    _descriptionRead = new Exif::SyncWidget( i18n("Fields to get value from"), hbox,
+            opt->defaultFields( false, QString::fromAscii("description")) );
+    _descriptionWrite = new Exif::SyncWidget( i18n("Fields to write value to"), hbox,
+            opt->defaultFields( true, QString::fromAscii("description")) );
+    _syncTabs->addTab( hbox, i18n("Description") );
+
+    hbox = new QHBox( _syncPage );
+    hbox->setSpacing( 6 );
+
+    _dateRead = new Exif::SyncWidget( i18n("Fields to get value from"), hbox,
+            opt->defaultFields( false, QString::fromAscii("date")) );
+    _dateWrite = new Exif::SyncWidget( i18n("Fields to write value to"), hbox,
+            opt->defaultFields( true, QString::fromAscii("date")) );
+    _syncTabs->addTab( hbox, i18n("Date") );
+
+    QValueList<DB::CategoryPtr> categories = DB::ImageDB::instance()->categoryCollection()->categories();
+    for( QValueList<DB::CategoryPtr>::Iterator it = categories.begin(); it != categories.end(); ++it )
+        if ( !(*it)->isSpecialCategory() )
+            slotCategoryAdded( (*it)->name() );
+
+    _lay->addWidget( _syncTabs );
+#endif
+}
+
+void Settings::SettingsDialog::slotCategoryAdded( const QString& name )
+{
+    for (int i = 0; _syncTabs->label( i ) != QString::null; ++i )
+        if ( _syncTabs->label( i ).remove( '&' ) == name ) {
+            qDebug("Attempted to add category \"%s\" that already exists, skipping...", name.ascii());
+            return;
+        }
+
+    QGrid* box = new QGrid( 2, Horizontal, _syncPage );
+    box->setSpacing( 6 );
+
+
+#ifdef HASEXIV2
+    const Settings::SettingsData* opt = Settings::SettingsData::instance();
+
+    _catFieldsRead.replace( name, new Exif::SyncWidget( i18n("Fields to get value from"), box,
+                opt->defaultFields( false, QString::fromAscii("category_%1").arg( name ) )) );
+    _catFieldsWrite.replace( name, new Exif::SyncWidget( i18n("Fields to write value to"), box,
+                opt->defaultFields( true, QString::fromAscii("category_%1").arg( name ) )) );
+    QLabel* lbl = new QLabel( i18n("Supercategories"), box );
+    lbl->setAlignment( AlignRight | AlignVCenter );
+    _catSuper.replace( name, new KComboBox( box ) );
+    _catSuper[ name ]->insertStringList( QStringList() <<
+            i18n("Treat all levels separately") << i18n("Include only the deepest level") <<
+            i18n("Join all levels by slash") );
+    lbl = new QLabel( i18n("Multiple values"), box );
+    lbl->setAlignment( AlignRight | AlignVCenter );
+    _catMulti.replace( name, new KComboBox( box ) );
+    _catMulti[ name ]->insertStringList( QStringList() <<
+            i18n("Repeat field") << i18n("Comma separated values") << i18n("Semicolon separated values") );
+    // create a dummy spacer
+    new QWidget( box );
+    _catAddName.replace( name, new QCheckBox( i18n("Include category name in tag value" ), box ) );
+    _syncTabs->addTab( box, name );
+#endif
+}
+
+void Settings::SettingsDialog::slotCategoryRemoved( const QString& name )
+{
+#ifdef HASEXIV2
+    for (int i = 0; _syncTabs->page( i ); ++i )
+        if ( _syncTabs->label( i ).remove( '&' ) == name ) {
+            _catFieldsRead.remove( name );
+            _catFieldsWrite.remove( name );
+            _catMulti.remove( name );
+            _catSuper.remove( name );
+            _catAddName.remove( name );
+            QWidget* tab = _syncTabs->page( i );
+            _syncTabs->removePage( tab );
+            delete tab;
+            return;
+        }
+#else
+    Q_UNUSED( name );
+#endif
+}
+
+void Settings::SettingsDialog::slotCategoryRenamed( const QString& oldName, const QString& newName )
+{
+    slotCategoryRemoved( oldName );
+    slotCategoryAdded( newName );
+}
+
 void Settings::SettingsDialog::showBackendPage()
 {
     showPage(_backendPageIndex);
