@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2005 Jesper K. Pedersen <blackie@kde.org>
+/* Copyright (C) 2003-2006 Jesper K. Pedersen <blackie@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -19,16 +19,16 @@
 #ifndef THUMBNAILBUILDER_H
 #define THUMBNAILBUILDER_H
 
-#include "imageclient.h"
+#include "ImageManager/ImageClient.h"
 #include <qprogressdialog.h>
 #include <qimage.h>
-#include "imageinfolist.h"
+#include "DB/ImageInfoList.h"
 
 
 namespace ThumbnailView
 {
 
-class ThumbnailBuilder :public QProgressDialog, public ImageClient {
+class ThumbnailBuilder :public QProgressDialog, public ImageManager::ImageClient {
     Q_OBJECT
 
 public:
@@ -39,7 +39,7 @@ public:
 private:
     QStringList _images;
     uint _index;
-    QMap<QString, ImageInfoPtr> _infoMap;
+    QMap<QString, DB::ImageInfoPtr> _infoMap;
 };
 
 }
