@@ -51,10 +51,12 @@ ThumbnailView::ThumbnailWidget* ThumbnailView::ThumbnailWidget::_instance = 0;
 ThumbnailView::ThumbnailWidget::ThumbnailWidget( QWidget* parent, const char* name )
     :QGridView( parent, name ),
      _gridResizeInteraction( this ),
+     _wheelResizing( false ),
      _selectionInteraction( this ),
      _mouseTrackingHandler( this ),
      _mouseHandler( &_mouseTrackingHandler ),
-     _sortDirection( Settings::SettingsData::instance()->showNewestThumbnailFirst() ? NewestFirst : OldestFirst )
+     _sortDirection( Settings::SettingsData::instance()->showNewestThumbnailFirst() ? NewestFirst : OldestFirst ),
+     _cellOnFirstShiftMovementKey(Cell::invalidCell())
 {
     _instance = this;
     setFocusPolicy( WheelFocus );
@@ -85,7 +87,7 @@ void ThumbnailView::ThumbnailWidget::paintCell( QPainter * p, int row, int col )
     QPixmap doubleBuffer( cellRect().size() );
     QPainter painter( &doubleBuffer );
     paintCellBackground( &painter, row, col );
-    if ( !_mouseHandler->isResizingGrid() ) {
+    if ( !_mouseHandler->isResizingGrid() && !_wheelResizing ) {
         paintCellPixmap( &painter, row, col );
         paintCellText( &painter, row, col );
     }
@@ -100,13 +102,11 @@ void ThumbnailView::ThumbnailWidget::paintCellPixmap( QPainter* painter, int row
 {
     QString fileName = fileNameInCell( row, col );
     if ( !fileName.isNull() ) {
-        QPixmap* pix = QPixmapCache::find( fileNameInCell( row, col ) );
+        QPixmap* pix = QPixmapCache::find( fileName );
         if ( pix ) {
             QRect rect = iconGeometry( row, col );
             Q_ASSERT( !rect.isNull() );
             painter->drawPixmap( rect, *pix );
-            if ( _selectedFiles.contains( fileName ) )
-                painter->fillRect( rect, QBrush( palette().active().highlight(), Dense4Pattern ) );
 
             rect = QRect( 0, 0, cellWidth(), cellHeight() );
             if ( _leftDrop == fileName )
@@ -116,9 +116,12 @@ void ThumbnailView::ThumbnailWidget::paintCellPixmap( QPainter* painter, int row
 
         }
         else {
-            int size = Settings::SettingsData::instance()->thumbSize();
+            QRect dimensions = cellDimensions();
             int angle = DB::ImageDB::instance()->info( fileName )->angle();
-            ThumbnailRequest* request = new ThumbnailRequest( fileName, QSize( size, size ), angle, this );
+            ThumbnailRequest* request = new ThumbnailRequest(
+                fileName, QSize( dimensions.width() - 2 * Settings::SettingsData::instance()->thumbnailSpace(),
+                                 dimensions.height() - 2 * Settings::SettingsData::instance()->thumbnailSpace() ),
+                angle, this );
             request->setCache();
             ImageManager::Manager::instance()->load( request );
         }
@@ -132,8 +135,10 @@ QString ThumbnailView::ThumbnailWidget::thumbnailText( const QString& fileName )
 {
     QString text, line;
 
-    int thumbnailSize = Settings::SettingsData::instance()->thumbSize();
-    int maxCharacters = thumbnailSize / QFontMetrics( font() ).maxWidth() * 2;
+    QRect dimensions = cellDimensions();
+    int thumbnailHeight = dimensions.height() - 2 * Settings::SettingsData::instance()->thumbnailSpace();
+    int thumbnailWidth = dimensions.width(); // no substracting here
+    int maxCharacters = thumbnailHeight / QFontMetrics( font() ).maxWidth() * 2;
 
     if ( Settings::SettingsData::instance()->displayLabels()) {
         text += DB::ImageDB::instance()->info( fileName )->label();;
@@ -142,15 +147,15 @@ QString ThumbnailView::ThumbnailWidget::thumbnailText( const QString& fileName )
 
     if ( Settings::SettingsData::instance()->displayCategories()) {
         QStringList grps = DB::ImageDB::instance()->info( fileName )->availableCategories();
-        for( QStringList::Iterator it = grps.begin(); it != grps.end(); ++it ) {
+        for( QStringList::const_iterator it = grps.constBegin(); it != grps.constEnd(); ++it ) {
             QString category = *it;
             if ( category != QString::fromLatin1( "Folder" ) && category != QString::fromLatin1( "Media Type" ) ) {
                 StringSet items = DB::ImageDB::instance()->info( fileName )->itemsOfCategory( category );
-                if (items.count() != 0 ) {
+                if (!items.empty()) {
                     line = QString::fromLatin1( "%1: " )
                            .arg( category );
                     bool first = true;
-                    for( StringSet::Iterator it2 = items.begin(); it2 != items.end(); ++it2 ) {
+                    for( StringSet::const_iterator it2 = items.begin(); it2 != items.end(); ++it2 ) {
                         QString item = *it2;
                         if ( first )
                             first = false;
@@ -158,7 +163,7 @@ QString ThumbnailView::ThumbnailWidget::thumbnailText( const QString& fileName )
                             line += QString::fromLatin1( ", " );
                         line += item;
                     }
-                    if ( QFontMetrics( font() ).width( line ) > thumbnailSize ) {
+                    if ( QFontMetrics( font() ).width( line ) > thumbnailWidth ) {
                         line = line.left( maxCharacters );
                         line += QString::fromLatin1( " ..." );
                     }
@@ -240,17 +245,47 @@ QString ThumbnailView::ThumbnailWidget::fileNameAtCoordinate( const QPoint& coor
     int row = rowAt( contentsPos.y() );
 
     QRect cellRect = const_cast<ThumbnailWidget*>(this)->cellGeometry( row, col );
-    QRect iconRect = iconGeometry( row, col );
 
-    // map iconRect from local coordinates within the cell to contents coordinates
-    iconRect.moveBy( cellRect.x(), cellRect.y() );
-
-    if ( iconRect.contains( contentsPos ) )
+    if ( cellRect.contains( contentsPos ) )
         return fileNameInCell( row, col );
     else
         return QString::null;
 }
 
+/**
+ * Return desired size of the whole cell
+ */
+QRect ThumbnailView::ThumbnailWidget::cellDimensions() const
+{
+    int width = Settings::SettingsData::instance()->thumbSize();
+    int height = width;
+
+    switch (Settings::SettingsData::instance()->thumbnailAspectRatio()) {
+        case Settings::Aspect_16_9:
+	    height = (int) (height * 9.0 / 16);
+	    break;
+        case Settings::Aspect_4_3:
+	    height = (int) (height * 3.0 / 4);
+	    break;
+        case Settings::Aspect_3_2:
+	    height = (int) (height * 2.0 / 3);
+	    break;
+        case Settings::Aspect_9_16:
+	    width = (int) (width * 9.0 / 16);
+	    break;
+        case Settings::Aspect_3_4:
+	    width = (int) (width * 3.0 / 4);
+	    break;
+        case Settings::Aspect_2_3:
+	    width = (int) (width * 2.0 / 3);
+	    break;
+	case Settings::Aspect_1_1:
+	    // nothing
+	    ;
+    }
+    return QRect(0, 0, width, height);
+}
+
 /**
  * Return the geometry for the icon in the cell (row,col). The returned coordinates are local to the cell.
  */
@@ -260,28 +295,64 @@ QRect ThumbnailView::ThumbnailWidget::iconGeometry( int row, int col ) const
     if ( fileName.isNull() ) // empty cell
         return QRect();
 
-    int size = Settings::SettingsData::instance()->thumbSize() + SPACE;
+    QRect dimensions = cellDimensions();
+    const int space = Settings::SettingsData::instance()->thumbnailSpace();
+    int width = dimensions.width() - 2 * space;
+    int height = dimensions.height() - 2 * space;
+
     QPixmap* pix = QPixmapCache::find( fileName );
     if ( !pix )
-        return QRect( SPACE, SPACE, size, size );
+        return QRect( space, space, width, height );
+
+    int xoff = space + (width - pix->width()) / 2;
+    int yoff = space + (height - pix->height()) / 2;
 
-    int xoff = 1 + (size - pix->width())/2; // 1 is for the border at the left
-    int yoff = (size - pix->height() );
-    if ( !Settings::SettingsData::instance()->displayLabels() && !Settings::SettingsData::instance()->displayCategories())
-        yoff /= 2; // we wil center the images if we do not show the label, otherwise we will align it to the bottom
     return QRect( xoff, yoff, pix->width(), pix->height() );
 }
 
+/**
+ * Returns the max. count of categories of an image with number of groups > 0
+ */
+int ThumbnailView::ThumbnailWidget::noOfCategoriesForImage(const QString& image ) const
+{
+    int catsInText = 0;
+    QStringList grps = DB::ImageDB::instance()->info( image )->availableCategories();
+    for( QStringList::const_iterator it = grps.constBegin(); it != grps.constEnd(); ++it ) {
+        QString category = *it;
+        if ( category != QString::fromLatin1( "Folder" ) && category != QString::fromLatin1( "Media Type" ) ) {
+            StringSet items = DB::ImageDB::instance()->info( image )->itemsOfCategory( category );
+            if (!items.empty()) {
+                catsInText++;
+            }
+        }
+    }
+    return catsInText;
+}
+
 /**
  * Return the height of the text under the thumbnails.
  */
-int ThumbnailView::ThumbnailWidget::textHeight() const
+int ThumbnailView::ThumbnailWidget::textHeight( bool reCalc ) const
 {
     int h = 0;
+    static int maxCatsInText = 0;
+
     if ( Settings::SettingsData::instance()->displayLabels() )
         h += QFontMetrics( font() ).height() +2;
     if ( Settings::SettingsData::instance()->displayCategories()) {
-        h += QFontMetrics( font() ).height() * ( DB::ImageDB::instance()->categoryCollection()->categoryNames().count() - 2 ) +2;
+        if ( reCalc ) {
+            if (!_selectedFiles.empty()) {
+                for( StringSet::const_iterator itImg = _selectedFiles.begin(); itImg != _selectedFiles.end(); ++itImg ) {
+                    maxCatsInText = QMAX( noOfCategoriesForImage( *itImg ), maxCatsInText );
+                }
+            } else {
+                maxCatsInText = 0;
+                for( QValueVector<QString>::ConstIterator itImg = _imageList.begin(); itImg != _imageList.end(); ++itImg ) {
+                    maxCatsInText = QMAX( noOfCategoriesForImage( *itImg ), maxCatsInText );
+                }
+            }
+        }
+        h += QFontMetrics( font() ).height() * ( maxCatsInText ) +5;
     }
     return h;
 }
@@ -296,7 +367,7 @@ QRect ThumbnailView::ThumbnailWidget::cellTextGeometry( int row, int col ) const
     if ( fileName.isNull() ) // empty cell
         return QRect();
 
-    int h = textHeight();
+    int h = textHeight( false );
 
     QRect iconRect = iconGeometry( row, col );
     QRect cellRect = const_cast<ThumbnailWidget*>(this)->cellGeometry( row, col );
@@ -376,18 +447,22 @@ void ThumbnailView::ThumbnailWidget::showEvent( QShowEvent* )
 void ThumbnailView::ThumbnailWidget::paintCellBackground( QPainter* p, int row, int col )
 {
     QRect rect = cellRect();
-    p->fillRect( rect, palette().active().base() );
+    if (_selectedFiles.contains(fileNameInCell(row, col)))
+        p->fillRect( rect, palette().active().highlight() );
+    else
+        p->fillRect( rect, palette().active().base() );
 
-    p->setPen( palette().active().dark() );
-    // left of frame
-    if ( col != 0 )
-        p->drawLine( rect.left(), rect.top(), rect.left(), rect.bottom() );
+    if (_mouseHandler->isResizingGrid() || _wheelResizing ||
+        Settings::SettingsData::instance()->thumbnailDisplayGrid()) {
+        p->setPen( palette().active().dark() );
+        // left of frame
+        if ( col != 0 )
+            p->drawLine( rect.left(), rect.top(), rect.left(), rect.bottom() );
 
-    // bottom line
-    if ( row != numRows() -1 ) {
-        p->drawLine( rect.left(), rect.bottom() -1, rect.right(), rect.bottom()-1 );
-        p->setPen( palette().active().light() );
-        p->drawLine( rect.left(), rect.bottom() -2, rect.right(), rect.bottom()-2 );
+        // bottom line
+        if ( row != numRows() -1 ) {
+            p->drawLine( rect.left(), rect.bottom() -1, rect.right(), rect.bottom()-1 );
+        }
     }
 }
 
@@ -395,16 +470,16 @@ void ThumbnailView::ThumbnailWidget::keyPressEvent( QKeyEvent* event )
 {
     if ( event->stateAfter() == 0 && event->state() == 0 && ( event->key() >= Key_A && event->key() <= Key_Z ) ) {
         QString token = event->text().upper().left(1);
-        bool firstHasToken = false;
-        bool firstTime     = true;
+        bool mustRemoveToken = false;
+        bool hadHit          = false;
 
-        for( Set<QString>::Iterator it = _selectedFiles.begin(); it != _selectedFiles.end(); ++it ) {
-            if ( firstTime ) {
-                firstHasToken = DB::ImageDB::instance()->info( *it )->hasCategoryInfo( QString::fromLatin1("Tokens"), token );
-                firstTime = false;
+        for( StringSet::const_iterator it = _selectedFiles.begin(); it != _selectedFiles.end(); ++it ) {
+            if ( ! hadHit ) {
+                mustRemoveToken = DB::ImageDB::instance()->info( *it )->hasCategoryInfo( QString::fromLatin1("Tokens"), token );
+                hadHit = true;
             }
 
-            if ( firstHasToken )
+            if ( mustRemoveToken )
                 DB::ImageDB::instance()->info( *it )->removeCategoryInfo( QString::fromLatin1("Tokens"), token );
             else
                 DB::ImageDB::instance()->info( *it )->addCategoryInfo( QString::fromLatin1("Tokens"), token );
@@ -412,6 +487,9 @@ void ThumbnailView::ThumbnailWidget::keyPressEvent( QKeyEvent* event )
             updateCell( *it );
         }
 
+        if ( hadHit )
+            updateCellSize();
+
         DB::ImageDB::instance()->categoryCollection()->categoryForName( QString::fromLatin1("Tokens") )->addItem( token );
         MainWindow::DirtyIndicator::markDirty();
     }
@@ -428,10 +506,21 @@ void ThumbnailView::ThumbnailWidget::keyPressEvent( QKeyEvent* event )
     possibleEmitSelectionChanged();
 }
 
+void ThumbnailView::ThumbnailWidget::keyReleaseEvent( QKeyEvent* event )
+{
+    if ( _wheelResizing && event->key() == Qt::Key_Control ) {
+        _wheelResizing = false;
+        repaintScreen();
+    }
+    else {
+        if ( event->key() == Qt::Key_Shift )
+            _cellOnFirstShiftMovementKey = Cell::invalidCell();
+        QGridView::keyReleaseEvent(event);
+    }
+}
+
 void ThumbnailView::ThumbnailWidget::keyboardMoveEvent( QKeyEvent* event )
 {
-    static QString startPossition;
-
     if ( !( event->state()& ShiftButton ) && !( event->state() &  ControlButton ) ) {
         clearSelection();
     }
@@ -443,6 +532,13 @@ void ThumbnailView::ThumbnailWidget::keyboardMoveEvent( QKeyEvent* event )
 
     // Update current position if it is outside view and we do not have any modifiers
     // that is if we just scroll arround.
+    //
+    // Use case is following: There is a selected item which is not
+    // visible because user has scrolled by other means than the
+    // keyboard (scrollbar or mouse wheel). In that case if the user
+    // presses keyboard movement key, the selection is forgotten and
+    // instead a currently visible cell is selected. So no scrolling
+    // of the view will be done.
     if ( !( event->state()& ShiftButton ) && !( event->state() &  ControlButton ) ) {
         if ( currentPos.row() < firstVisibleRow( PartlyVisible ) )
             currentPos = Cell( firstVisibleRow( FullyVisible ), currentPos.col() );
@@ -502,9 +598,19 @@ void ThumbnailView::ThumbnailWidget::keyboardMoveEvent( QKeyEvent* event )
     if ( newPos < Cell(0,0) )
         newPos = Cell(0,0);
 
-    // Update focus cell, and set selection
-    if ( (event->state() & ShiftButton) && !startPossition.isEmpty() )
-        selectItems( positionForFileName( startPossition ), newPos );
+    if ( event->state() & ShiftButton ) {
+        if ( _cellOnFirstShiftMovementKey == Cell::invalidCell() ) {
+            _cellOnFirstShiftMovementKey = currentPos;
+            _selectionOnFirstShiftMovementKey = _selectedFiles;
+        }
+
+        StringSet oldSelection = _selectedFiles;
+
+        _selectedFiles = _selectionOnFirstShiftMovementKey;
+        selectAllCellsBetween( _cellOnFirstShiftMovementKey, newPos, false );
+
+        repaintAfterChangedSelection( oldSelection );
+    }
 
     if ( ! (event->state() & ControlButton ) ) {
         selectCell( newPos );
@@ -512,9 +618,6 @@ void ThumbnailView::ThumbnailWidget::keyboardMoveEvent( QKeyEvent* event )
     }
     _currentItem = fileNameInCell( newPos );
 
-    if ( !( event->state() & ShiftButton ) || startPossition.isEmpty() )
-        startPossition = _currentItem;
-
     // Scroll if necesary
     if ( newPos.row() > lastVisibleRow( ThumbnailWidget::FullyVisible ) )
         setContentsPos( contentsX(), cellGeometry( newPos.row(), newPos.col() ).top() -
@@ -529,22 +632,31 @@ void ThumbnailView::ThumbnailWidget::keyboardMoveEvent( QKeyEvent* event )
  */
 void ThumbnailView::ThumbnailWidget::selectItems( const Cell& start, const Cell& end )
 {
-    Set<QString> oldSelection = _selectedFiles;
+    StringSet oldSelection = _selectedFiles;
+
     _selectedFiles.clear();
 
     selectAllCellsBetween( start, end, false );
 
-    for( Set<QString>::Iterator it = oldSelection.begin(); it != oldSelection.end(); ++it ) {
+    repaintAfterChangedSelection(oldSelection);
+}
+
+/**
+ * Repaint cells that are in different state than in given selection.
+ */
+void ThumbnailView::ThumbnailWidget::repaintAfterChangedSelection( const StringSet& oldSelection )
+{
+    for( StringSet::const_iterator it = oldSelection.begin(); it != oldSelection.end(); ++it ) {
         if ( !_selectedFiles.contains( *it ) )
             updateCell( *it );
     }
 
-    for( Set<QString>::Iterator it = _selectedFiles.begin(); it != _selectedFiles.end(); ++it ) {
+    for( StringSet::const_iterator it = _selectedFiles.begin(); it != _selectedFiles.end(); ++it ) {
         if ( !oldSelection.contains( *it ) )
             updateCell( *it );
     }
-
 }
+
 /**
  * Return the number of complete rows per page
  */
@@ -555,7 +667,8 @@ int ThumbnailView::ThumbnailWidget::numRowsPerPage() const
 
 void ThumbnailView::ThumbnailWidget::mousePressEvent( QMouseEvent* event )
 {
-    if (event->button() & MidButton )
+    if ( (event->button() & MidButton) ||
+         ((event->state() & Qt::ControlButton) && (event->state() & Qt::AltButton)) )
         _mouseHandler = &_gridResizeInteraction;
     else
         _mouseHandler = &_selectionInteraction;
@@ -583,6 +696,22 @@ void ThumbnailView::ThumbnailWidget::mouseDoubleClickEvent( QMouseEvent * event
     }
 }
 
+void ThumbnailView::ThumbnailWidget::wheelEvent( QWheelEvent* event )
+{
+    if ( event->state() & ControlButton ) {
+        event->accept();
+
+        _wheelResizing = true;
+
+        int delta = event->delta() / 20;
+
+        Settings::SettingsData::instance()->setThumbSize( QMAX( 32, cellWidth() + delta ) );
+
+        updateCellSize();
+    }
+    else
+        QGridView::wheelEvent(event);
+}
 
 
 void ThumbnailView::ThumbnailWidget::emitDateChange( int x, int y )
@@ -732,9 +861,9 @@ void ThumbnailView::ThumbnailWidget::resizeEvent( QResizeEvent* e )
 
 void ThumbnailView::ThumbnailWidget::clearSelection()
 {
-    Set<QString> oldSelection = _selectedFiles;
+    StringSet oldSelection = _selectedFiles;
     _selectedFiles.clear();
-    for( Set<QString>::Iterator fileIt = oldSelection.begin(); fileIt != oldSelection.end(); ++fileIt ) {
+    for( StringSet::const_iterator fileIt = oldSelection.begin(); fileIt != oldSelection.end(); ++fileIt ) {
         updateCell( *fileIt );
     }
 }
@@ -772,7 +901,7 @@ bool ThumbnailView::ThumbnailWidget::isMovementKey( int key )
 void ThumbnailView::ThumbnailWidget::toggleSelection( const QString& fileName )
 {
     if ( _selectedFiles.contains( fileName ) )
-        _selectedFiles.remove( fileName );
+        _selectedFiles.erase( fileName );
     else
         _selectedFiles.insert( fileName );
 
@@ -795,7 +924,7 @@ QStringList ThumbnailView::ThumbnailWidget::selection( bool keepSortOrderOfDatab
 
 void ThumbnailView::ThumbnailWidget::possibleEmitSelectionChanged()
 {
-    static Set<QString> oldSelection;
+    static StringSet oldSelection;
     if ( oldSelection != _selectedFiles ) {
         oldSelection = _selectedFiles;
         emit selectionChanged();
@@ -822,17 +951,38 @@ void ThumbnailView::ThumbnailWidget::selectAll()
     repaintScreen();
 }
 
-void ThumbnailView::ThumbnailWidget::reload(bool flushCache )
+void ThumbnailView::ThumbnailWidget::reload(bool flushCache, bool clearSelection)
 {
     if ( flushCache )
         QPixmapCache::clear();
-    _selectedFiles.clear();
+    if ( clearSelection )
+        _selectedFiles.clear();
     updateCellSize();
     repaintScreen();
 }
 
 void ThumbnailView::ThumbnailWidget::repaintScreen()
 {
+    if ( Settings::SettingsData::instance()->thumbnailDarkBackground() ) {
+        setPaletteBackgroundColor( Qt::black );
+        setPaletteForegroundColor( Qt::white );
+
+        QPalette p(palette());
+        QColor c = p.color(QPalette::Active, QColorGroup::Highlight);
+        if ((c.red() < 0x30 && c.green() < 0x30 && c.blue() < 0x30) ||
+            (c.red() > 0xd0 && c.green() > 0xd0 && c.blue() > 0xd0)) {
+            // Not enough contrast to bg or fg. Use light blue instead.
+            static QColor highlightColor(0x67, 0x8d, 0xb2);
+            p.setColor(QPalette::Active, QColorGroup::Highlight, highlightColor);
+            p.setColor(QPalette::Inactive, QColorGroup::Highlight, highlightColor);
+            p.setColor(QPalette::Disabled, QColorGroup::Highlight, highlightColor);
+            setPalette(p);
+        }
+    }
+    else {
+        unsetPalette();  // fallback to default.
+    }
+
     for ( int row = firstVisibleRow( PartlyVisible ); row <= lastVisibleRow( PartlyVisible ); ++row )
         for ( int col = 0; col < numCols(); ++col )
             QGridView::repaintCell( row, col );
@@ -969,10 +1119,10 @@ void ThumbnailView::ThumbnailWidget::ensureCellsSorted( Cell& pos1, Cell& pos2 )
 
 void ThumbnailView::ThumbnailWidget::slotRepaint()
 {
-    if ( (int) _pendingRepaint.count() > numCols() * numRowsPerPage() / 2 )
+    if ( (int) _pendingRepaint.size() > numCols() * numRowsPerPage() / 2 )
         repaintScreen();
     else {
-        for( Set<QString>::Iterator it = _pendingRepaint.begin(); it != _pendingRepaint.end(); ++it ) {
+        for( StringSet::const_iterator it = _pendingRepaint.begin(); it != _pendingRepaint.end(); ++it ) {
             Cell cell = positionForFileName( *it );
             QGridView::repaintCell( cell.row(), cell.col() );
         }
@@ -1025,12 +1175,17 @@ QValueVector<QString> ThumbnailView::ThumbnailWidget::reverseVector( const QValu
 
 void ThumbnailView::ThumbnailWidget::updateCellSize()
 {
-    int size = Settings::SettingsData::instance()->thumbSize() + SPACE;
-    setCellWidth( size );
+    QRect dimensions = cellDimensions();
+    setCellWidth( dimensions.width() );
 
-    int h = size +2 + textHeight();
-    setCellHeight( h );
+    const int oldHeight = cellHeight();
+    const int height = dimensions.height() + 2 + textHeight( true );
+    setCellHeight( height );
     updateGridSize();
+    if ( height != oldHeight && ! _currentItem.isNull() ) {
+        const Cell c = positionForFileName(_currentItem);
+        ensureCellVisible( c.row(), c.col() );
+    }
 }
 
 void ThumbnailView::ThumbnailWidget::viewportPaintEvent( QPaintEvent* e )
