@@ -34,9 +34,9 @@
 #include "MainWindow/Window.h"
 #include "X11/X.h"
 
+#include "Utilities/JpeglibWithFix.h"
+
 extern "C" {
-#define XMD_H // prevent INT32 clash from jpeglib
-#include <jpeglib.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -55,6 +55,14 @@ extern "C" {
 #endif
 
 #include <kdebug.h>
+#include <qtextcodec.h>
+#include <kmdcodec.h>
+
+namespace {
+    QString _localEncoding = i18n("Local encoding");
+    /* cache for humanReadableCharsetList() */
+    QStringList _charsetMapping( _localEncoding );
+}
 
 /**
  * Given an ImageInfoPtr this function will create an HTML blob about the
@@ -86,13 +94,19 @@ QString Utilities::createInfoText( DB::ImageInfoPtr info, QMap< int,QPair<QStrin
 
     if ( Settings::SettingsData::instance()->showImageSize() && info->mediaType() == DB::Image)  {
         const QSize imageSize = info->size();
-        const double megapix = imageSize.width() * imageSize.height() / 1000000.0;
-        text += i18n("<b>Image Size: </b> ") +
+        // Do not add -1 x -1 text
+        if (imageSize.width() >= 0 && imageSize.height() >= 0) {
+            const double megapix = imageSize.width() * imageSize.height() / 1000000.0;
+            text += i18n("<b>Image Size: </b> ") +
                 QString::number(imageSize.width()) + i18n("x") +
-                QString::number(imageSize.height()) +
-                QString::fromLatin1(" (") + QString::number(megapix, 'f', 1) +
-                i18n("MP") + QString::fromLatin1(")") +
-                linebreak;
+                QString::number(imageSize.height());
+            if (megapix > 0.05) {
+                text +=
+                    QString::fromLatin1(" (") + QString::number(megapix, 'f', 1) +
+                    i18n("Short for Mega Pixels", "MP") + QString::fromLatin1(")");
+            }
+            text += linebreak;
+        }
     }
 
     QValueList<DB::CategoryPtr> categories = DB::ImageDB::instance()->categoryCollection()->categories();
@@ -101,10 +115,10 @@ QString Utilities::createInfoText( DB::ImageInfoPtr info, QMap< int,QPair<QStrin
         const QString categoryName = (*categoryIt)->name();
         if ( (*categoryIt)->doShow() ) {
             const StringSet items = info->itemsOfCategory( categoryName );
-            if (items.count() != 0 ) {
+            if (!items.empty()) {
                 text += QString::fromLatin1( "<b>%1: </b> " ).arg( (*categoryIt)->text() );
                 bool first = true;
-                for( StringSet::ConstIterator it2 = items.begin(); it2 != items.end(); ++it2 ) {
+                for( StringSet::const_iterator it2 = items.begin(); it2 != items.end(); ++it2 ) {
                     QString item = *it2;
                     if ( first )
                         first = false;
@@ -124,6 +138,11 @@ QString Utilities::createInfoText( DB::ImageInfoPtr info, QMap< int,QPair<QStrin
         }
     }
 
+    if ( Settings::SettingsData::instance()->showLabel() && !info->label().isEmpty())  {
+        if ( !text.isEmpty() )
+            text += i18n("<b>Label: </b> ") +  info->label() + linebreak;
+    }
+
     if ( Settings::SettingsData::instance()->showDescription() && !info->description().isEmpty())  {
         if ( !text.isEmpty() )
             text += i18n("<b>Description: </b> ") +  info->description() + linebreak;
@@ -132,9 +151,29 @@ QString Utilities::createInfoText( DB::ImageInfoPtr info, QMap< int,QPair<QStrin
 #ifdef HASEXIV2
     QString exifText;
     if ( Settings::SettingsData::instance()->showEXIF() ) {
-        QMap<QString,QString> exifMap = Exif::Info::instance()->infoForViewer( info->fileName() );
-        for( QMap<QString,QString>::Iterator exifIt = exifMap.begin(); exifIt != exifMap.end(); ++exifIt ) {
-            exifText += QString::fromLatin1( "<b>%1: </b> %2<br>" ).arg( exifIt.key() ).arg( exifIt.data() );
+        QMap<QString,QStringList> exifMap = Exif::Info::instance()->infoForViewer( info->fileName(), true );
+        // at first, put there only EXIF data
+        for( QMap<QString,QStringList>::Iterator exifIt = exifMap.begin(); exifIt != exifMap.end(); ++exifIt ) {
+            if ( exifIt.key().startsWith( QString::fromLatin1( "Exif." ) ) )
+                for ( QStringList::const_iterator valuesIt = exifIt.data().begin(); valuesIt != exifIt.data().end(); ++valuesIt )
+                    exifText += QString::fromLatin1( "<b>%1: </b> %2<br>" ).arg( 
+                            QStringList::split( QString::fromLatin1("."), exifIt.key() ).last()
+                        ).arg( *valuesIt );
+        }
+
+        QString iptcText;
+        for( QMap<QString,QStringList>::Iterator exifIt = exifMap.begin(); exifIt != exifMap.end(); ++exifIt ) {
+            if ( !exifIt.key().startsWith( QString::fromLatin1( "Exif." ) ) )
+                for ( QStringList::const_iterator valuesIt = exifIt.data().begin(); valuesIt != exifIt.data().end(); ++valuesIt )
+                    iptcText += QString::fromLatin1( "<b>%1: </b> %2<br>" ).arg( 
+                            QStringList::split( QString::fromLatin1("."), exifIt.key() ).last()
+                        ).arg( *valuesIt );
+        }
+        if ( !iptcText.isEmpty() ) {
+            if ( exifText.isEmpty() )
+                exifText = iptcText;
+            else
+                exifText += QString::fromLatin1( "<hr>" ) + iptcText;
         }
     }
 
@@ -208,7 +247,7 @@ QString Utilities::setupDemo()
     }
 
     // index.xml
-    QString str = readInstalledFile( QString::fromLatin1( "demo/index.xml" ) );
+    QString str = readFile(locateDataFile(QString::fromLatin1("demo/index.xml")));
     if ( str.isNull() )
         exit(-1);
 
@@ -228,7 +267,7 @@ QString Utilities::setupDemo()
     }
 
     // exif-info.db
-    QString fileName = locate( "data", QString::fromLatin1( "kphotoalbum/demo/exif-info.db" ) );
+    QString fileName = locateDataFile(QString::fromLatin1("demo/exif-info.db"));
     if ( !fileName.isEmpty() )
         copy( fileName, dir + QString::fromLatin1( "/exif-info.db" ) );
 
@@ -287,32 +326,13 @@ bool Utilities::makeHardLink( const QString& from, const QString& to )
         return true;
 }
 
-QString Utilities::readInstalledFile( const QString& fileName )
+QString Utilities::getThumbnailDir( const QString& imageFile )
 {
-    QString inFileName = locate( "data", QString::fromLatin1( "kphotoalbum/%1" ).arg( fileName ) );
-    if ( inFileName.isEmpty() ) {
-        KMessageBox::error( 0, i18n("<p>Unable to find kphotoalbum/%1. This is likely an installation error. Did you remember to do a 'make install'? Did you set KDEDIRS, in case you did not install it in the default location?</p>").arg( fileName ) ); // Proof reader comment: What if it was a binary installation? (eg. apt-get)
-        return QString::null;
-    }
-
-    QFile file( inFileName );
-    if ( !file.open( IO_ReadOnly ) ) {
-        KMessageBox::error( 0, i18n("Could not open file %1.").arg( inFileName ) );
-        return QString::null;
-    }
-
-    QTextStream stream( &file );
-    QString content = stream.read();
-    file.close();
-
-    return content;
-}
-
-QString Utilities::getThumbnailDir( const QString& imageFile ) {
     return QFileInfo( imageFile ).dirPath() + QString::fromLatin1("/ThumbNails");
 }
 
-QString Utilities::getThumbnailFile( const QString& imageFile, int width, int height, int angle ) {
+QString Utilities::getThumbnailFile( const QString& imageFile, int width, int height, int angle )
+{
     QFileInfo info( imageFile );
     while (angle < 0)
         angle += 360;
@@ -345,6 +365,12 @@ bool Utilities::canReadImage( const QString& fileName )
 }
 
 
+QString Utilities::locateDataFile(const QString& fileName)
+{
+    return
+        locate("data", QString::fromLatin1("kphotoalbum/") + fileName);
+}
+
 QString Utilities::readFile( const QString& fileName )
 {
     if ( fileName.isEmpty() ) {
@@ -384,6 +410,11 @@ extern "C"
     }
 }
 
+namespace Utilities
+{
+    bool loadJPEG(QImage *img, FILE* inputFile, QSize* fullSize, int dim );
+}
+
 bool Utilities::loadJPEG(QImage *img, const QString& imageFile, QSize* fullSize, int dim)
 {
     FILE* inputFile=fopen( QFile::encodeName(imageFile), "rb");
@@ -482,26 +513,6 @@ bool Utilities::isJPEG( const QString& fileName )
     return format == QString::fromLocal8Bit( "JPEG" );
 }
 
-QStringList Utilities::shuffle( const QStringList& input )
-{
-    QStringList list = input;
-    static bool init = false;
-    if ( !init ) {
-        QTime midnight( 0, 0, 0 );
-        srand( midnight.secsTo(QTime::currentTime()) );
-        init = true;
-    }
-
-    QStringList result;
-
-    while ( list.count() != 0 ) {
-        int index = (int) ( (double)list.count()* rand()/((double)RAND_MAX) );
-        result.append( list[index] );
-        list.remove( list[index] );
-    }
-    return result;
-}
-
 /**
    Create a maping from original name with path to uniq name without:
    cd1/def.jpg      -> def.jpg
@@ -551,12 +562,14 @@ Utilities::UniqNameMap Utilities::createUniqNameMap( const QStringList& images,
     return map;
 }
 
-QString Utilities::normalizedFileName( const QString& fileName )
+namespace Utilities
+{
+QString normalizedFileName( const QString& fileName )
 {
     return QFileInfo(fileName).absFilePath();
 }
 
-QString Utilities::dereferenceSymLinks( const QString& fileName )
+QString dereferenceSymLinks( const QString& fileName )
 {
     QFileInfo fi(fileName);
     int rounds = 256;
@@ -566,6 +579,7 @@ QString Utilities::dereferenceSymLinks( const QString& fileName )
         return QString::null;
     return fi.filePath();
 }
+}
 
 bool Utilities::areSameFile( const QString fileName1, const QString fileName2 )
 {
@@ -631,16 +645,6 @@ QString Utilities::stripImageDirectory( const QString& fileName )
         return fileName;
 }
 
-QStringList Utilities::diff( const QStringList& list1, const QStringList& list2 )
-{
-    QStringList result;
-    for( QStringList::ConstIterator it = list1.constBegin(); it != list1.constEnd(); ++it ) {
-        if ( !list2.contains( *it ) )
-            result.append( *it );
-    }
-    return result;
-}
-
 QString Utilities::absoluteImageFileName( const QString& relativeName )
 {
     return stripSlash( Settings::SettingsData::instance()->imageDirectory() ) + QString::fromLatin1( "/" ) + relativeName;
@@ -658,7 +662,7 @@ bool operator<( const QPoint& p1, const QPoint& p2)
 bool Utilities::isVideo( const QString& fileName )
 {
     static StringSet videoExtensions;
-    if ( videoExtensions.isEmpty() ) {
+    if ( videoExtensions.empty() ) {
         videoExtensions.insert( QString::fromLatin1( "3gp" ) );
         videoExtensions.insert( QString::fromLatin1( "avi" ) );
         videoExtensions.insert( QString::fromLatin1( "mp4" ) );
@@ -681,6 +685,7 @@ bool Utilities::isVideo( const QString& fileName )
         videoExtensions.insert( QString::fromLatin1( "wmv" ) );
         videoExtensions.insert( QString::fromLatin1( "ogm" ) );
         videoExtensions.insert( QString::fromLatin1( "rm" ) );
+        videoExtensions.insert( QString::fromLatin1( "flv" ) );
     }
 
     QFileInfo fi( fileName );
@@ -704,16 +709,72 @@ QImage Utilities::scaleImage(const QImage &image, const QSize& s, QImage::ScaleM
     return image.scale(s, mode);
 }
 
-QStringList Utilities::removeDuplicates( const QStringList& items )
+/**
+ * Convert a weird C string in some ugly encoding to a cute unicode QString
+ */
+QString Utilities::cStringWithEncoding( const char *c_str, const QString& charset )
 {
-    Set<QString> seen;
-    QStringList res;
-    for( QStringList::ConstIterator itemIt = items.begin(); itemIt != items.end(); ++itemIt ) {
-        if ( !seen.contains( *itemIt ) ) {
-            res.append( *itemIt );
-            seen.insert( *itemIt );
+    if ( charset == _localEncoding )
+        return QString::fromLocal8Bit( c_str );
+    QTextCodec* codec = QTextCodec::codecForName( charset.ascii() );
+    if (!codec)
+        return QString::fromLocal8Bit( c_str );
+    return codec->toUnicode( c_str );
+}
+
+QString Utilities::cStringWithEncoding( const char *c_str, int charset ) 
+{
+    if ( charset < 0 )
+        charset = 0;
+    else if ( charset >= static_cast<int>( Utilities::humanReadableCharsetList().size() ) )
+        charset = 0;
+    return Utilities::cStringWithEncoding( c_str, Utilities::humanReadableCharsetList()[ charset ] );
+}
+
+std::string Utilities::encodeQString( const QString& str, const QString& charset ) 
+{
+    if ( charset == _localEncoding )
+        return std::string( str.local8Bit() );
+
+    QTextCodec* codec = QTextCodec::codecForName( charset.ascii() );
+    if (!codec)
+        return std::string( str.local8Bit() );
+    return codec->fromUnicode( str ).data();
+}
+
+std::string Utilities::encodeQString( const QString& str, int charset )
+{
+    if ( charset < 0 )
+        charset = 0;
+    else if ( charset >= static_cast<int>( Utilities::humanReadableCharsetList().size() ) )
+        charset = 0;
+    return Utilities::encodeQString( str, Utilities::humanReadableCharsetList()[ charset ] );
+}
+
+/**
+ * Return a list of all encodings that Qt3 supports
+ */
+QStringList Utilities::humanReadableCharsetList()
+{
+    if ( _charsetMapping.count() == 1 )
+        for (int mib = 3; mib < 3000; ++mib) {
+            // Why 3..2999? See http://www.iana.org/assignments/character-sets
+            QTextCodec* codec = QTextCodec::codecForMib( mib );
+            if ( codec && (codec->mibEnum() == mib) )
+                _charsetMapping.append( QString::fromLocal8Bit( codec->name() ) );
         }
-    }
-    return res;
+    return _charsetMapping;
+}
+
+DB::MD5 Utilities::MD5Sum( const QString& fileName )
+{
+    QFile file( fileName );
+    if ( !file.open( IO_ReadOnly ) )
+        return DB::MD5();
+
+    KMD5 md5calculator( 0 /* char* */);
+    md5calculator.reset();
+    md5calculator.update( file );
+    return DB::MD5(md5calculator.hexDigest());
 }
 
