@@ -29,6 +29,7 @@
 #include <math.h>
 #include "DB/ImageDB.h"
 #include "ImageDisplay.h"
+#include <qtimer.h>
 
 /**
    Area displaying the actual image in the viewer.
@@ -82,7 +83,8 @@
 */
 
 Viewer::ImageDisplay::ImageDisplay( QWidget* parent, const char* name )
-    :Display( parent, name ), _reloadImageInProgress( false ), _forward(true), _curIndex(0),_busy( false )
+    :Display( parent, name ), _reloadImageInProgress( false ), _forward(true), _curIndex(0),_busy( false ),
+    _cursorHiding(true)
 {
     setBackgroundMode( NoBackground );
 
@@ -96,10 +98,51 @@ Viewer::ImageDisplay::ImageDisplay( QWidget* parent, const char* name )
     // This is to ensure that people do see the drawing when they draw,
     // otherwise the drawing would disappear as soon as mouse was released.
     connect( _drawHandler, SIGNAL( active() ), this, SLOT( doShowDrawings() ) );
+
+    setMouseTracking( true );
+    _cursorTimer = new QTimer( this );
+    connect( _cursorTimer, SIGNAL( timeout() ), this, SLOT( hideCursor() ) );
+    showCursor();
+
+}
+
+/**
+ * If mouse cursor hiding is enabled, hide the cursor right now
+ */
+void Viewer::ImageDisplay::hideCursor() {
+    if (_cursorHiding)
+        setCursor( Qt::blankCursor );
+}
+
+/**
+ * If mouse cursor hiding is enabled, show normal cursor and start a timer that will hide it later
+ */
+void Viewer::ImageDisplay::showCursor() {
+    if (_cursorHiding) {
+        unsetCursor();
+        _cursorTimer->start( 1500, true );
+    }
+}
+
+/**
+ * Prevent hideCursor() and showCursor() from altering cursor state
+ */
+void Viewer::ImageDisplay::disableCursorHiding() {
+    _cursorHiding = false;
+}
+
+/**
+ * Enable automatic mouse cursor hiding
+ */
+void Viewer::ImageDisplay::enableCursorHiding() {
+    _cursorHiding = true;
 }
 
 void Viewer::ImageDisplay::mousePressEvent( QMouseEvent* event )
 {
+    // disable cursor hiding till button release
+    disableCursorHiding();
+
     QMouseEvent e( event->type(), mapPos( event->pos() ), event->button(), event->state() );
     double ratio = sizeRatio( QSize(_zEnd.x()-_zStart.x(), _zEnd.y()-_zStart.y()), size() );
     bool block = _currentHandler->mousePressEvent( &e, event->pos(), ratio );
@@ -110,6 +153,9 @@ void Viewer::ImageDisplay::mousePressEvent( QMouseEvent* event )
 
 void Viewer::ImageDisplay::mouseMoveEvent( QMouseEvent* event )
 {
+    // just reset the timer
+    showCursor();
+
     QMouseEvent e( event->type(), mapPos( event->pos() ), event->button(), event->state() );
     double ratio = sizeRatio( QSize(_zEnd.x()-_zStart.x(), _zEnd.y()-_zStart.y()), size() );
     bool block = _currentHandler->mouseMoveEvent( &e, event->pos(), ratio );
@@ -120,6 +166,10 @@ void Viewer::ImageDisplay::mouseMoveEvent( QMouseEvent* event )
 
 void Viewer::ImageDisplay::mouseReleaseEvent( QMouseEvent* event )
 {
+    // enable cursor hiding and reset timer
+    enableCursorHiding();
+    showCursor();
+
     _cache.remove( _curIndex );
     QMouseEvent e( event->type(), mapPos( event->pos() ), event->button(), event->state() );
     double ratio = sizeRatio( QSize(_zEnd.x()-_zStart.x(), _zEnd.y()-_zStart.y()), size() );
@@ -149,6 +199,7 @@ void Viewer::ImageDisplay::drawAll()
 
 void Viewer::ImageDisplay::startDrawing()
 {
+    disableCursorHiding();
     _currentHandler = _drawHandler;
 }
 
@@ -157,6 +208,7 @@ void Viewer::ImageDisplay::stopDrawing()
     _drawHandler->stopDrawing();
     _currentHandler = _viewHandler;
     drawAll();
+    showCursor();
 }
 
 void Viewer::ImageDisplay::toggleShowDrawings( bool b )
@@ -183,9 +235,11 @@ bool Viewer::ImageDisplay::setImage( DB::ImageInfoPtr info, bool forward )
         _loadedImage = found->img;
         updateZoomPoints( Settings::SettingsData::instance()->viewerStandardSize(), found->img.size() );
         cropAndScale();
+        info->setSize( found->size );
+        emit imageReady();
     }
     else {
-        requestImage( info );
+        requestImage( info, true );
         busy();
     }
     _forward = forward;
@@ -225,10 +279,10 @@ void Viewer::ImageDisplay::paintEvent( QPaintEvent* )
     int y = ( height() - _viewPixmap.height() ) / 2;
     bitBlt( this, x, y, &_viewPixmap );
     QPainter p( this );
-    p.fillRect( 0,0, width(), y, black ); // top
-    p.fillRect( 0,height()-y, width(), height()-y, black ); // bottom
-    p.fillRect( 0,0, x, height(), black ); // left
-    p.fillRect( width()-x, 0, width()-x, height(), black ); // right
+    p.fillRect( 0, 0, width(), y, black ); // top
+    p.fillRect( 0, height()-y-1, width(), height()-y, black ); // bottom
+    p.fillRect( 0, 0, x, height(), black ); // left
+    p.fillRect( width()-x-1, 0, width()-x, height(), black ); // right
 }
 
 QPoint Viewer::ImageDisplay::offset( int logicalWidth, int logicalHeight, int physicalWidth, int physicalHeight, double* ratio )
@@ -406,6 +460,7 @@ void Viewer::ImageDisplay::pixmapLoaded( const QString& fileName, const QSize& i
 
         _loadedImage = img;
         cropAndScale();
+        emit imageReady();
     }
     else {
         if ( imgSize != size() )
@@ -573,7 +628,7 @@ double Viewer::ImageDisplay::sizeRatio( const QSize& baseSize, const QSize& newS
     return res;
 }
 
-void Viewer::ImageDisplay::requestImage( const DB::ImageInfoPtr& info )
+void Viewer::ImageDisplay::requestImage( const DB::ImageInfoPtr& info, bool priority )
 {
     Settings::StandardViewSize viewSize = Settings::SettingsData::instance()->viewerStandardSize();
     QSize s = size();
@@ -582,6 +637,8 @@ void Viewer::ImageDisplay::requestImage( const DB::ImageInfoPtr& info )
 
     ImageManager::ImageRequest* request = new ImageManager::ImageRequest( info->fileName(), s, info->angle(), this );
     request->setUpScale( viewSize == Settings::FullSize );
+    if ( priority )
+        request->setPriority();
     ImageManager::Manager::instance()->load( request );
 }
 
